<!-- CSHashTable.xslt -->
<!-- Written by Saar Carmi [saar@bigfoot.com] -->
<!-- Updated by Matthias Hess [matthias.hess@bluewin.ch] to fix XSLT "garbage" problem -->
<!-- Updated by Tor-Erik Hagen [tor-erik@web-amp.com] to make it possible not to specify namespace -->
<!-- Updated by James Johnson [jtjohnson@takklesoft.com] to expose a typesafe enumerator from GetEnumerator -->
<xsl:transform version='1.0' xmlns:xsl='http://www.w3.org/1999/XSL/Transform'>
  <xsl:output method='text'/>
  
  <xsl:template match="/">
	<xsl:apply-templates select='/typeSafeCollections/typeSafeCollection[templateKind="HashTable"]'/>
  </xsl:template>
  
  <xsl:template match='/typeSafeCollections/typeSafeCollection[templateKind="HashTable"]'>

<xsl:if test="collectionNamespace">
	<xsl:if test="string-length(collectionNamespace) > 0">
namespace <xsl:value-of select='collectionNamespace' />
{
	</xsl:if>
</xsl:if>

<xsl:choose>
  <xsl:when test="accessibility"><xsl:value-of select='accessibility' /></xsl:when>
	<xsl:otherwise>public </xsl:otherwise>
</xsl:choose>
	class <xsl:value-of select="collectionName"/> : IDictionary, ICollection, IEnumerable, ICloneable
	{
		protected Hashtable innerHash;
		
		#region "Constructors"
		public  <xsl:value-of select="collectionName"/>()
		{
			innerHash = new Hashtable();
		}
		public <xsl:value-of select="collectionName"/>(<xsl:value-of select="collectionName"/> original)
		{
			innerHash = new Hashtable (original.innerHash);
		}
		public <xsl:value-of select="collectionName"/>(IDictionary dictionary)
		{
			innerHash = new Hashtable (dictionary);
		}

		public <xsl:value-of select="collectionName"/>(int capacity)
		{
			innerHash = new Hashtable(capacity);
		}

		public <xsl:value-of select="collectionName"/>(IDictionary dictionary, float loadFactor)
		{
			innerHash = new Hashtable(dictionary, loadFactor);
		}

		public <xsl:value-of select="collectionName"/>(IHashCodeProvider codeProvider, IComparer comparer)
		{
			innerHash = new Hashtable (codeProvider, comparer);
		}

		public <xsl:value-of select="collectionName"/>(int capacity, int loadFactor)
		{
			innerHash = new Hashtable(capacity, loadFactor);
		}

		public <xsl:value-of select="collectionName"/>(IDictionary dictionary, IHashCodeProvider codeProvider, IComparer comparer)
		{
			innerHash = new Hashtable (dictionary, codeProvider, comparer);
		}
		
		public <xsl:value-of select="collectionName"/>(int capacity, IHashCodeProvider codeProvider, IComparer comparer)
		{
			innerHash = new Hashtable (capacity, codeProvider, comparer);
		}

		public <xsl:value-of select="collectionName"/>(IDictionary dictionary, float loadFactor, IHashCodeProvider codeProvider, IComparer comparer)
		{
			innerHash = new Hashtable (dictionary, loadFactor, codeProvider, comparer);
		}

		public <xsl:value-of select="collectionName"/>(int capacity, float loadFactor, IHashCodeProvider codeProvider, IComparer comparer)
		{
			innerHash = new Hashtable (capacity, loadFactor, codeProvider, comparer);
		}

		
#endregion

		#region Implementation of IDictionary
        public <xsl:value-of select="collectionName"/>Enumerator GetEnumerator()
        {
	        return new <xsl:value-of select="collectionName"/>Enumerator(this);
        }
        
		System.Collections.IDictionaryEnumerator IDictionary.GetEnumerator()
		{
			return new <xsl:value-of select="collectionName"/>Enumerator(this);
		}
		
		IEnumerator IEnumerable.GetEnumerator()
		{
			return GetEnumerator();
		}

		public void Remove(<xsl:value-of select="keyType"/> key)
		{
			innerHash.Remove (key);
		}
		void IDictionary.Remove(object key)
		{
			Remove ((<xsl:value-of select="keyType"/>)key);
		}

		public bool Contains(<xsl:value-of select="keyType"/> key)
		{
			return innerHash.Contains(key);
		}
		bool IDictionary.Contains(object key)
		{
			return Contains((<xsl:value-of select="keyType"/>)key);
		}

		public void Clear()
		{
			innerHash.Clear();		
		}

		public void Add(<xsl:value-of select="keyType"/> key, <xsl:value-of select="itemType"/> value)
		{
			innerHash.Add (key, value);
		}
		void IDictionary.Add(object key, object value)
		{
			Add ((<xsl:value-of select="keyType"/>)key, (<xsl:value-of select="itemType"/>)value);
		}

		public bool IsReadOnly
		{
			get
			{
				return innerHash.IsReadOnly;
			}
		}

		public <xsl:value-of select="itemType"/> this[<xsl:value-of select="keyType"/> key]
		{
			get
			{
				return (<xsl:value-of select="itemType"/>) innerHash[key];
			}
			set
			{
				innerHash[key] = value;
			}
		}
		object IDictionary.this[object key]
		{
			get
			{
				return this[(<xsl:value-of select="keyType"/>)key];
			}
			set
			{
				this[(<xsl:value-of select="keyType"/>)key] = (<xsl:value-of select="itemType"/>)value;
			}
		}
        
		public System.Collections.ICollection Values
		{
			get
			{
				return innerHash.Values;
			}
		}

		public System.Collections.ICollection Keys
		{
			get
			{
				return innerHash.Keys;
			}
		}

		public bool IsFixedSize
		{
			get
			{
				return innerHash.IsFixedSize;
			}
		}
		#endregion

		#region Implementation of ICollection
		public void CopyTo(System.Array array, int index)
		{
			innerHash.CopyTo (array, index);
		}

		public bool IsSynchronized
		{
			get
			{
				return innerHash.IsSynchronized;
			}
		}

		public int Count
		{
			get
			{
				return innerHash.Count;
			}
		}

		public object SyncRoot
		{
			get
			{
				return innerHash.SyncRoot;
			}
		}
		#endregion

		#region Implementation of ICloneable
		public <xsl:value-of select="collectionName"/> Clone()
		{
			<xsl:value-of select="collectionName"/> clone = new <xsl:value-of select="collectionName"/>();
			clone.innerHash = (Hashtable) innerHash.Clone();
			
			return clone;
		}
		object ICloneable.Clone()
		{
			return Clone();
		}
		#endregion
		
		#region "HashTable Methods"
		public bool ContainsKey (<xsl:value-of select="keyType"/> key)
		{
			return innerHash.ContainsKey(key);
		}
		public bool ContainsValue (<xsl:value-of select="itemType"/> value)
		{
			return innerHash.ContainsValue(value);
		}
		public static <xsl:value-of select="collectionName"/> Synchronized(<xsl:value-of select="collectionName"/> nonSync)
		{
			<xsl:value-of select="collectionName"/> sync = new <xsl:value-of select="collectionName"/>();
			sync.innerHash = Hashtable.Synchronized(nonSync.innerHash);

			return sync;
		}
		#endregion

		internal Hashtable InnerHash
		{
			get
			{
				return innerHash;
			}
		}
	}
	
	public class <xsl:value-of select="collectionName"/>Enumerator : IDictionaryEnumerator
	{
		private IDictionaryEnumerator innerEnumerator;
			
		internal <xsl:value-of select="collectionName"/>Enumerator (<xsl:value-of select="collectionName"/> enumerable)
		{
			innerEnumerator = enumerable.InnerHash.GetEnumerator();
		}

		#region Implementation of IDictionaryEnumerator
		public <xsl:value-of select="keyType"/> Key
		{
			get
			{
				return (<xsl:value-of select="keyType"/>)innerEnumerator.Key;
			}
		}
		object IDictionaryEnumerator.Key
		 {
			 get
			 {
				 return Key;
			 }
		 }


		public <xsl:value-of select="itemType"/> Value
		{
			get
			{
				return (<xsl:value-of select="itemType"/>)innerEnumerator.Value;
			}
		}
		object IDictionaryEnumerator.Value
		{
			get
			{
				return Value;
			}
		}

		public System.Collections.DictionaryEntry Entry
		{
			get
			{
				return innerEnumerator.Entry;
			}
		}

		#endregion

		#region Implementation of IEnumerator
		public void Reset()
		{
			innerEnumerator.Reset();
		}

		public bool MoveNext()
		{
			return innerEnumerator.MoveNext();
		}

		public object Current
		{
			get
			{
				return innerEnumerator.Current;
			}
		}
		#endregion
	}
<xsl:if test="collectionNamespace">
	<xsl:if test="string-length(collectionNamespace) > 0">
}
	</xsl:if>
</xsl:if>

  </xsl:template>

</xsl:transform>

  