<!-- CSSortedList.xslt -->
<!-- This code leveraged from Joel Mueller [jmueller@swiftk.com] -->
<!-- Updated by Matthias Hess [matthias.hess@bluewin.ch] to fix XSLT "garbage" problem -->
<!-- Updated by Tor-Erik Hagen [tor-erik@web-amp.com] to make it possible not to specify namespace -->
<xsl:transform version='1.0' xmlns:xsl='http://www.w3.org/1999/XSL/Transform'>
  <xsl:output method='text'/>
  
  <xsl:template match="/">
	<xsl:apply-templates select='/typeSafeCollections/typeSafeCollection[templateKind="SortedList"]'/>
  </xsl:template>

  <xsl:template match='/typeSafeCollections/typeSafeCollection[templateKind="SortedList"]'>

<xsl:if test="collectionNamespace">
	<xsl:if test="string-length(collectionNamespace) > 0">
namespace <xsl:value-of select='collectionNamespace' />
{
	</xsl:if>
</xsl:if>

	/// &lt;summary&gt;
	///		Represents a strongly-typed collection of key-and-value pairs that are
	///		sorted by the keys and are accessible by key and by index.
	/// &lt;/summary&gt;
	/// &lt;seealso cref="System.Collections.SortedList"/&gt;
	[Serializable]
<xsl:choose>
  <xsl:when test="accessibility"><xsl:value-of select='accessibility' /></xsl:when>
	<xsl:otherwise>public </xsl:otherwise>
</xsl:choose>
  class <xsl:value-of select='collectionName'/> : IDictionary, ICloneable
	{
		#region Member Variables
		private const int DEFAULT_CAPACITY = 16;

		private <xsl:value-of select='keyType' />[] keys;
		private <xsl:value-of select='itemType' />[] values;
		private int count;
		[NonSerialized]
		private int version;
		private IComparer comparer;
		private KeyList keyList;
		private ValueList valueList;
		#endregion

		#region Constructors
		/// &lt;summary&gt;
		///		Initializes a new instance of the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; class that is empty, 
		///		has the default initial capacity and is sorted according to the &lt;see cref="IComparable"/&gt; interface 
		///		implemented by each key added to the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.
		/// &lt;/summary&gt;
		public <xsl:value-of select='collectionName'/>()
		{
			keys = new <xsl:value-of select='keyType' />[DEFAULT_CAPACITY];
			values = new <xsl:value-of select='itemType' />[DEFAULT_CAPACITY];
			comparer = Comparer.Default;
		}

		/// &lt;summary&gt;
		///		Initializes a new instance of the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; class that is empty, 
		///		has the specified initial capacity and is sorted according to the &lt;see cref="IComparable"/&gt;
		///		interface implemented by each key added to the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.
		/// &lt;/summary&gt;
		/// &lt;param name="capacity"&gt;The initial number of elements that the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; can contain.&lt;/param&gt;
		/// &lt;exception cref="ArgumentOutOfRangeException"&gt;&lt;paramref name="capacity"/&gt; is less than zero.&lt;/exception&gt;
		public <xsl:value-of select='collectionName'/>(int capacity)
		{
			if (capacity &lt; 0)
				throw new ArgumentOutOfRangeException("capacity", capacity, "Initial capacity cannot be less than zero.");

			keys = new <xsl:value-of select='keyType' />[capacity];
			values = new <xsl:value-of select='itemType' />[capacity];
			comparer = Comparer.Default;
		}

		/// &lt;summary&gt;
		///		Initializes a new instance of the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; class that is empty, 
		///		has the default initial capacity and is sorted according to the specified 
		///		&lt;see cref="IComparer"/&gt; interface.
		/// &lt;/summary&gt;
		/// &lt;param name="comparer"&gt;
		///		&lt;para&gt;The &lt;see cref="IComparer"/&gt; implementation to use when comparing keys.&lt;/para&gt;
		///		&lt;para&gt;-or-&lt;/para&gt;
		///		&lt;para&gt;A null reference, to use the &lt;see cref="IComparable"/&gt; implementation of each key.&lt;/para&gt;
		/// &lt;/param&gt;
		public <xsl:value-of select='collectionName'/>(IComparer comparer) : this()
		{
			if (comparer != null)
				this.comparer = comparer;
		}

		/// &lt;summary&gt;
		///		Initializes a new instance of the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; class that is empty, 
		///		has the specified initial capacity and is sorted according to the specified 
		///		&lt;see cref="IComparer"/&gt; interface.
		/// &lt;/summary&gt;
		/// &lt;param name="comparer"&gt;
		///		&lt;para&gt;The &lt;see cref="IComparer"/&gt; implementation to use when comparing keys.&lt;/para&gt;
		///		&lt;para&gt;-or-&lt;/para&gt;
		///		&lt;para&gt;A null reference, to use the &lt;see cref="IComparable"/&gt; implementation of each key.&lt;/para&gt;
		/// &lt;/param&gt;
		/// &lt;param name="capacity"&gt;The initial number of elements that the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; can contain.&lt;/param&gt;
 		/// &lt;exception cref="ArgumentOutOfRangeException"&gt;&lt;paramref name="capacity"/&gt; is less than zero.&lt;/exception&gt;
		public <xsl:value-of select='collectionName'/>(IComparer comparer, int capacity) : this(capacity)
		{
			if (comparer != null)
				this.comparer = comparer;
		}

		/// &lt;summary&gt;
		///		Initializes a new instance of the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; class that contains 
		///		elements copied from the specified dictionary, has the same initial capacity as the 
		///		number of elements copied and is sorted according to the &lt;see cref="IComparable"/&gt; interface 
		///		implemented by each key.
		/// &lt;/summary&gt;
		/// &lt;param name="d"&gt;The &lt;see cref="IDictionary"/&gt; to copy to a new SortedList.&lt;/param&gt;
		/// &lt;exception cref="ArgumentNullException"&gt;&lt;paramref name="d"/&gt; is a null reference.&lt;/exception&gt;
		/// &lt;exception cref="InvalidCastException"&gt;
		///		&lt;para&gt;One or more elements in &lt;paramref name="d"/&gt; do not implement the
		///		&lt;see cref="IComparable"/&gt; interface.&lt;/para&gt;
		///		&lt;para&gt;-or-&lt;/para&gt;
		///		&lt;para&gt;&lt;pararef name="d"/&gt; contains elements of a type not supported by &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.&lt;/para&gt;
		///	&lt;/exception&gt;
		public <xsl:value-of select='collectionName'/>(IDictionary d) : this(d, null) {}

		/// &lt;summary&gt;
		///		Initializes a new instance of the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; class that contains 
		///		elements copied from the specified dictionary, has the same initial capacity as the 
		///		number of elements copied and is sorted according to the specified &lt;see cref="IComparer"/&gt; interface.
		/// &lt;/summary&gt;
		/// &lt;param name="d"&gt;The &lt;see cref="IDictionary"/&gt; to copy to a new SortedList.&lt;/param&gt;
		/// &lt;param name="comparer"&gt;
		///		&lt;para&gt;The &lt;see cref="IComparer"/&gt; implementation to use when comparing keys.&lt;/para&gt;
		///		&lt;para&gt;-or-&lt;/para&gt;
		///		&lt;para&gt;A null reference, to use the &lt;see cref="IComparable"/&gt; implementation of each key.&lt;/para&gt;
		/// &lt;/param&gt;
		/// &lt;exception cref="ArgumentNullException"&gt;&lt;paramref name="d"/&gt; is a null reference.&lt;/exception&gt;
		/// &lt;exception cref="InvalidCastException"&gt;
		///		&lt;para&gt;One or more elements in &lt;paramref name="d"/&gt; do not implement the
		///		&lt;see cref="IComparable"/&gt; interface.&lt;/para&gt;
		///		&lt;para&gt;-or-&lt;/para&gt;
		///		&lt;para&gt;&lt;pararef name="d"/&gt; contains elements of a type not supported by &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.&lt;/para&gt;
		///	&lt;/exception&gt;
		public <xsl:value-of select='collectionName'/>(IDictionary d, IComparer comparer)
			: this(comparer, (d == null ? 0 : d.Count))
		{
			if (d == null)
				throw new ArgumentNullException("d", "The IDictionary cannot be null.");

			d.Keys.CopyTo(keys, 0);
			d.Values.CopyTo(values, 0);
			Array.Sort(this.keys, this.values, this.comparer);
			this.count = d.Count;
		}
		#endregion

		#region Properties
		/// &lt;summary&gt;
		///		Gets and sets the value associated with a specific key in the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.
		/// &lt;/summary&gt;
		/// &lt;value&gt;
		///		The &lt;see cref="<xsl:value-of select='itemType' />"/&gt; associated with &lt;paramref name="key"/&gt; in the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;,
		///		if &lt;paramref name="key"/&gt; is found; otherwise, a &lt;see cref="NullReferenceException"/&gt; is thrown.
		///	&lt;/value&gt;
		/// &lt;remarks&gt;
		///		This property, unlike its equivalent in &lt;see cref="System.Collections.SortedList"/&gt;, does not return
		///		null when a key cannot be found. The reason for this is that the strongly-typed sorted list
		///		template this class is created from allows the value to be a value type (or struct), which cannot
		///		be set to null. Instead, a &lt;see cref="NullReferenceException"/&gt; is thrown when a key cannot be found.
		///		Callers must either catch this exception, or call the &lt;see cref="<xsl:value-of select='collectionName'/>.Contains"/&gt; method first to
		///		see if the key exists.
		/// &lt;/remarks&gt;
		/// &lt;exception cref="ArgumentNullException"&gt;The &lt;paramref name="key"/&gt; is a null reference.&lt;/exception&gt;
		/// &lt;exception cref="ArgumentException"&gt;
		///		Either the &lt;paramref name="key"/&gt; or the &lt;paramref name="value"/&gt; are not of a type supported by the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.
		/// &lt;/exception&gt;
		/// &lt;exception cref="NullReferenceException"&gt;
		///		&lt;paramref name="key"/&gt; is not found in the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.
		/// &lt;/exception&gt;
		object IDictionary.this[object key]
		{
			get
			{
				if (key == null)
					throw new ArgumentNullException("key", "The key cannot be null.");

				if (!(key is <xsl:value-of select='keyType' />))
					throw new ArgumentException("The key must be of type: " + typeof(<xsl:value-of select='keyType' />).FullName, "key");

				return this[(<xsl:value-of select='keyType' />)key];
			}
			set
			{
				if (key == null)
					throw new ArgumentNullException("key", "The key cannot be null.");

				if (!(key is <xsl:value-of select='keyType' />))
					throw new ArgumentException("The key must be of type: " + typeof(<xsl:value-of select='keyType' />).FullName, "key");

				if (!(value is <xsl:value-of select='itemType' />))
					throw new ArgumentException("The value must be of type: " + typeof(<xsl:value-of select='itemType' />).FullName, "value");

				this[(<xsl:value-of select='keyType' />)key] = (<xsl:value-of select='itemType' />)value;
			}
		}

		/// &lt;summary&gt;
		///		Gets and sets the value associated with a specific key in the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.
		/// &lt;/summary&gt;
		/// &lt;value&gt;
		///		The &lt;see cref="<xsl:value-of select='itemType' />"/&gt; associated with &lt;paramref name="key"/&gt; in the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;,
		///		if &lt;paramref name="key"/&gt; is found; otherwise, a &lt;see cref="NullReferenceException"/&gt; is thrown.
		///	&lt;/value&gt;
		/// &lt;remarks&gt;
		///		&lt;para&gt;This property, unlike its equivalent in &lt;see cref="System.Collections.SortedList"/&gt;, does not return
		///		null when a key cannot be found. The reason for this is that the strongly-typed sorted list
		///		template this class is created from allows the value to be a value type (or struct), which cannot
		///		be set to null. Instead, a &lt;see cref="NullReferenceException"/&gt; is thrown when a key cannot be found.
		///		Callers must either catch this exception, or call the &lt;see cref="<xsl:value-of select='collectionName'/>.Contains"/&gt; method first to
		///		see if the key exists.&lt;/para&gt;
		/// &lt;/remarks&gt;
		/// &lt;exception cref="ArgumentNullException"&gt;The &lt;paramref name="key"/&gt; is a null reference.&lt;/exception&gt;
		/// &lt;exception cref="NullReferenceException"&gt;
		///		&lt;paramref name="key"/&gt; is not found in the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.
		/// &lt;/exception&gt;
		public virtual <xsl:value-of select='itemType' /> this[<xsl:value-of select='keyType' /> key]
		{
			get
			{
				int index = IndexOfKey(key);
				if (index &gt;= 0)
					return values[index];

				throw new NullReferenceException("The specified key could not be found.");
			}
			set
			{
				if (Object.ReferenceEquals(key, null)) // avoids compiler error for null check on value type
					throw new ArgumentNullException("key", "The key cannot be null.");

				int index = Array.BinarySearch(keys, 0, count, key, comparer);
				if (index &gt;= 0)
				{
					values[index] = value;
					version++;
					return;
				}

				Insert(~index, key, value);
			}
		}

		/// &lt;summary&gt;
		///		Gets or sets the capacity of the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.
		/// &lt;/summary&gt;
		/// &lt;value&gt;The number of elements that the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; can contain.&lt;/value&gt;
		public virtual int Capacity
		{
			get
			{
				return keys.Length;
			}
			set
			{
				if (value &lt; count)
					value = count;

				if (value != keys.Length)
				{
					if (value &gt; 0)
					{
						<xsl:value-of select='keyType' />[] newKeys = new <xsl:value-of select='keyType' />[value];
						<xsl:value-of select='itemType' />[] newValues = new <xsl:value-of select='itemType' />[value];

						if (count &gt; 0)
						{
							Array.Copy(keys, 0, newKeys, 0, count);
							Array.Copy(values, 0, newValues, 0, count);
						}
						
						keys = newKeys;
						values = newValues;
					}
					else
					{
						keys = new <xsl:value-of select='keyType' />[DEFAULT_CAPACITY];
						values = new <xsl:value-of select='itemType' />[DEFAULT_CAPACITY];
					}
				}
			}
		}

		/// &lt;summary&gt;
		///		Gets the number of elements contained in the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.
		/// &lt;/summary&gt;
		/// &lt;value&gt;The number of elements contained in the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.&lt;/value&gt;
		public virtual int Count
		{
			get
			{
				return this.count;
			}
		}

		/// &lt;summary&gt;
		///		Gets a value indicating whether the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt; has a fixed size.
		/// &lt;/summary&gt;
		/// &lt;value&gt;
		///		&lt;b&gt;true&lt;/b&gt; if the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; has a fixed size; otherwise, &lt;b&gt;false&lt;/b&gt;.
		///		The default is &lt;b&gt;false&lt;/b&gt;.
		/// &lt;/value&gt;
		public virtual bool IsFixedSize
		{
			get { return false; }
		}

		/// &lt;summary&gt;
		///		Gets a value indicating whether the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt; is read-only.
		/// &lt;/summary&gt;
		/// &lt;value&gt;
		///		&lt;b&gt;true&lt;/b&gt; if the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; is read-only; otherwise, &lt;b&gt;false&lt;/b&gt;.
		///		The default is &lt;b&gt;false&lt;/b&gt;.
		/// &lt;/value&gt;
		public virtual bool IsReadOnly
		{
			get	{ return false; }
		}

		/// &lt;summary&gt;
		///		Gets a value indicating whether access to the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt; is
		///		synchronized (thread-safe).
		/// &lt;/summary&gt;
		/// &lt;value&gt;
		///		&lt;b&gt;true&lt;/b&gt; if access to the the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; is 
		///		synchronized (thread-safe); otherwise, &lt;b&gt;false&lt;/b&gt;.
		///		The default is &lt;b&gt;false&lt;/b&gt;.
		/// &lt;/value&gt;
		public virtual bool IsSynchronized
		{
			get { return false; }
		}

		/// &lt;summary&gt;
		///		Gets the keys in the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.
		/// &lt;/summary&gt;
		/// &lt;value&gt;An &lt;see cref="ICollection"/&gt; containing the keys in the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.&lt;/value&gt;
		/// &lt;remarks&gt;
		///		&lt;para&gt;The &lt;see cref="ICollection"/&gt; is a read-only view of the keys in the 
		///		&lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;. Modifications made to the underlying &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;
		///		are immediately reflected in the &lt;b&gt;ICollection&lt;/b&gt;.&lt;/para&gt;
		///		&lt;para&gt;The elements of the &lt;b&gt;ICollection&lt;/b&gt; are sorted in the same order as the keys
		///		of the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.&lt;/para&gt;
		///		&lt;para&gt;Similar to &lt;see cref="<xsl:value-of select='collectionName'/>.GetKeyList"/&gt;, but returns an &lt;b&gt;ICollection&lt;/b&gt;
		///		instead of an &lt;see cref="IList"/&gt;.&lt;/para&gt;
		/// &lt;/remarks&gt;
		public virtual ICollection Keys
		{
			get
			{
				return GetKeyList();
			}
		}

		/// &lt;summary&gt;
		///		Gets an object that can be used to synchronize access to the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.
		/// &lt;/summary&gt;
		/// &lt;value&gt;An object that can be used to synchronize access to the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.&lt;/value&gt;
		public virtual object SyncRoot
		{
			get { return this; }
		}

		/// &lt;summary&gt;
		///		Gets the values in the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.
		/// &lt;/summary&gt;
		/// &lt;value&gt;An &lt;see cref="ICollection"/&gt; containing the values in the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.&lt;/value&gt;
		/// &lt;remarks&gt;
		///		&lt;para&gt;The &lt;see cref="ICollection"/&gt; is a read-only view of the values in the 
		///		&lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;. Modifications made to the underlying &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;
		///		are immediately reflected in the &lt;b&gt;ICollection&lt;/b&gt;.&lt;/para&gt;
		///		&lt;para&gt;The elements of the &lt;b&gt;ICollection&lt;/b&gt; are sorted in the same order as the values
		///		of the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.&lt;/para&gt;
		///		&lt;para&gt;Similar to &lt;see cref="<xsl:value-of select='collectionName'/>.GetValueList"/&gt;, but returns an &lt;b&gt;ICollection&lt;/b&gt;
		///		instead of an &lt;see cref="IList"/&gt;.&lt;/para&gt;
		/// &lt;/remarks&gt;
		public virtual ICollection Values
		{
			get
			{
				return GetValueList();
			}
		}
		#endregion

		#region Public Methods
		/// &lt;summary&gt;
		///		Adds an element with the specified key and value to the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.
		/// &lt;/summary&gt;
		/// &lt;param name="key"&gt;The key of the element to add.&lt;/param&gt;
		/// &lt;param name="value"&gt;The value of the element to add.&lt;/param&gt;
		/// &lt;exception cref="ArgumentNullException"&gt;The &lt;paramref name="key"/&gt; is a null reference.&lt;/exception&gt;
		/// &lt;exception cref="ArgumentException"&gt;
		///		&lt;para&gt;An element with the specified &lt;paramref name="key"/&gt; already exists in the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.&lt;/para&gt;
		///		&lt;para&gt;-or-&lt;/para&gt;
		///		&lt;para&gt;Either the &lt;paramref name="key"/&gt; or the &lt;paramref name="value"/&gt; are not of a type supported by the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.&lt;/para&gt;
		///		&lt;para&gt;-or-&lt;/para&gt;
		///		&lt;para&gt;The &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt; is set to use the &lt;see cref="IComparable"/&gt; interface,
		///		and &lt;paramref name="key"/&gt; does not implement the &lt;b&gt;IComparable&lt;/b&gt; interface.&lt;/para&gt;
		/// &lt;/exception&gt;
		/// &lt;exception cref="InvalidOperationException"&gt;The comparer throws an exception.&lt;/exception&gt;
		/// &lt;exception cref="NotSupportedException"&gt;
		///		&lt;para&gt;The &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; is read-only.&lt;/para&gt;
		///		&lt;para&gt;-or-&lt;/para&gt;
		///		&lt;para&gt;The &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt; has a fixed size.&lt;/para&gt;
		/// &lt;/exception&gt;
		void IDictionary.Add(object key, object value)
		{
			if (key == null)
				throw new ArgumentNullException("key", "The key cannot be null.");

			if (!(key is <xsl:value-of select='keyType' />))
				throw new ArgumentException("The key must be of type: " + typeof(<xsl:value-of select='keyType' />).FullName, "key");

			if (!(value is <xsl:value-of select='itemType' />))
				throw new ArgumentException("The value must be of type: " + typeof(<xsl:value-of select='itemType' />).FullName, "value");

			this.Add((<xsl:value-of select='keyType' />)key, (<xsl:value-of select='itemType' />)value);
		}

		/// &lt;summary&gt;
		///		Adds an element with the specified key and value to the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.
		/// &lt;/summary&gt;
		/// &lt;param name="key"&gt;The key of the element to add.&lt;/param&gt;
		/// &lt;param name="value"&gt;The value of the element to add.&lt;/param&gt;
 		/// &lt;exception cref="ArgumentNullException"&gt;The &lt;paramref name="key"/&gt; is a null reference.&lt;/exception&gt;
		/// &lt;exception cref="ArgumentException"&gt;
		///		&lt;para&gt;An element with the specified &lt;paramref name="key"/&gt; already exists in the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.&lt;/para&gt;
		///		&lt;para&gt;-or-&lt;/para&gt;
		///		&lt;para&gt;The &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt; is set to use the &lt;see cref="IComparable"/&gt; interface,
		///		and &lt;paramref name="key"/&gt; does not implement the &lt;b&gt;IComparable&lt;/b&gt; interface.&lt;/para&gt;
		/// &lt;/exception&gt;
		/// &lt;exception cref="InvalidOperationException"&gt;The comparer throws an exception.&lt;/exception&gt;
		/// &lt;exception cref="NotSupportedException"&gt;
		///		&lt;para&gt;The &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; is read-only.&lt;/para&gt;
		///		&lt;para&gt;-or-&lt;/para&gt;
		///		&lt;para&gt;The &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt; has a fixed size.&lt;/para&gt;
		/// &lt;/exception&gt;
		public virtual void Add(<xsl:value-of select='keyType' /> key, <xsl:value-of select='itemType' /> value)
		{
			if (Object.ReferenceEquals(key, null)) // avoids compiler error for null check on value type
				throw new ArgumentNullException("key", "The key cannot be null.");

			int index = Array.BinarySearch(keys, 0, count, key, comparer);

			if (index &gt;= 0)
				throw new ArgumentException(String.Format("Item has already been added.  Key being added: \"{0}\".", key));
 
			Insert(~index, key, value);
		}

		/// &lt;summary&gt;
		///		Removes all elements from the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.
		/// &lt;/summary&gt;
		/// &lt;exception cref="NotSupportedException"&gt;
		///		&lt;para&gt;The &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; is read-only.&lt;/para&gt;
		///		&lt;para&gt;-or-&lt;/para&gt;
		///		&lt;para&gt;The &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt; has a fixed size.&lt;/para&gt;
		/// &lt;/exception&gt;
		public virtual void Clear()
		{
			keys = new <xsl:value-of select='keyType' />[DEFAULT_CAPACITY];
			values = new <xsl:value-of select='itemType' />[DEFAULT_CAPACITY];
			count = 0;
			version++;
		}

		/// &lt;summary&gt;
		///		Creates a shallow copy of the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.
		/// &lt;/summary&gt;
		/// &lt;returns&gt;A shallow copy of the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.&lt;/returns&gt;
		public virtual object Clone()
		{
			<xsl:value-of select='collectionName'/> newList = new <xsl:value-of select='collectionName'/>(count);
			Array.Copy(keys, 0, newList.keys, 0, count);
			Array.Copy(values, 0, newList.values, 0, count);
			newList.count = count;
			newList.version = version;
			newList.comparer = comparer;

			return newList;
		}

		/// &lt;summary&gt;
		///		Determines whether the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt; contains a specific key.
		/// &lt;/summary&gt;
		/// &lt;param name="key"&gt;The key to locate in the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.&lt;/param&gt;
		/// &lt;exception cref="ArgumentNullException"&gt;The &lt;paramref name="key"/&gt; is a null reference.&lt;/exception&gt;
		/// &lt;exception cref="ArgumentException"&gt;
		///		The &lt;paramref name="key"/&gt; is not of a type supported by the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.
		/// &lt;/exception&gt;
		/// &lt;exception cref="InvalidOperationException"&gt;The comparer throws an exception.&lt;/exception&gt;
		/// &lt;returns&gt;
		///		&lt;b&gt;true&lt;/b&gt; if the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; contains an element with the specified key; 
		///		otherwise, &lt;b&gt;false&lt;/b&gt;.
		///	&lt;/returns&gt;
		bool IDictionary.Contains(object key)
		{
			if (key == null)
				throw new ArgumentNullException("key", "The key cannot be null.");

			if (!(key is <xsl:value-of select='keyType' />))
				throw new ArgumentException("The key must be of type: " + typeof(<xsl:value-of select='keyType' />).FullName, "key");

			return (IndexOfKey((<xsl:value-of select='keyType' />)key) &gt;= 0);
		}

		/// &lt;summary&gt;
		///		Determines whether the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt; contains a specific key.
		/// &lt;/summary&gt;
		/// &lt;param name="key"&gt;The key to locate in the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.&lt;/param&gt;
		/// &lt;exception cref="ArgumentNullException"&gt;The &lt;paramref name="key"/&gt; is a null reference.&lt;/exception&gt;
		/// &lt;exception cref="InvalidOperationException"&gt;The comparer throws an exception.&lt;/exception&gt;
		/// &lt;returns&gt;
		///		&lt;b&gt;true&lt;/b&gt; if the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; contains an element with the specified 
		///		&lt;paramref name="key"/&gt;; otherwise, &lt;b&gt;false&lt;/b&gt;.
		///	&lt;/returns&gt;
		public virtual bool Contains(<xsl:value-of select='keyType' /> key)
		{
			return (IndexOfKey(key) &gt;= 0);
		}

		/// &lt;summary&gt;
		///		Determines whether the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt; contains a specific key.
		/// &lt;/summary&gt;
		/// &lt;param name="key"&gt;The key to locate in the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.&lt;/param&gt;
		/// &lt;exception cref="ArgumentNullException"&gt;The &lt;paramref name="key"/&gt; is a null reference.&lt;/exception&gt;
		/// &lt;exception cref="InvalidOperationException"&gt;The comparer throws an exception.&lt;/exception&gt;
		/// &lt;returns&gt;
		///		&lt;b&gt;true&lt;/b&gt; if the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; contains an element with the specified 
		///		&lt;paramref name="key"/&gt;; otherwise, &lt;b&gt;false&lt;/b&gt;.
		///	&lt;/returns&gt;
		public virtual bool ContainsKey(<xsl:value-of select='keyType' /> key)
		{
			return (IndexOfKey(key) &gt;= 0);
		}

		/// &lt;summary&gt;
		///		Determines whether the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt; contains a specific value.
		/// &lt;/summary&gt;
		/// &lt;param name="value"&gt;The value to locate in the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.&lt;/param&gt;
		/// &lt;returns&gt;
		///		&lt;b&gt;true&lt;/b&gt; if the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; contains an element with the specified 
		///		&lt;paramref name="value"/&gt;; otherwise, &lt;b&gt;false&lt;/b&gt;.
		/// &lt;/returns&gt;
		public virtual bool ContainsValue(<xsl:value-of select='itemType' /> value)
		{
			return (IndexOfValue(value) &gt;= 0);
		}

		/// &lt;summary&gt;
		///		Copies the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; elements to a one-dimensional &lt;see cref="System.Array"/&gt;
		///		instance at the specified index.
		/// &lt;/summary&gt;
		/// &lt;param name="array"&gt;
		///		The one-dimensional &lt;see cref="System.Array"/&gt; that is the destination of the
		///		&lt;see cref="DictionaryEntry"/&gt; objects copied from &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.
		///		The array must have zero-based indexing.
		/// &lt;/param&gt;
		/// &lt;param name="arrayIndex"&gt;The zero-based index in &lt;paramref name="array"/&gt; at which copying begins.&lt;/param&gt;
		/// &lt;exception cref="ArgumentNullException"&gt;&lt;paramref name="array"/&gt; is a null reference.&lt;/exception&gt;
		/// &lt;exception cref="ArgumentOutOfRangeException"&gt;&lt;paramref name="arrayIndex"/&gt; is less than zero.&lt;/exception&gt;
		/// &lt;exception cref="ArgumentException"&gt;
		///		&lt;para&gt;&lt;paramref name="array"/&gt; is multidimensional.&lt;/para&gt;
		///		&lt;para&gt;-or-&lt;/para&gt;
		///		&lt;para&gt;&lt;paramref name="arrayIndex"/&gt; is equal to or greater than the length of &lt;paramref name="array"/&gt;.&lt;/para&gt;
		///		&lt;para&gt;-or-&lt;/para&gt;
		///		&lt;para&gt;The number of elements in the source &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; is greater than the 
		///		available space from &lt;paramref name="arrayIndex"/&gt; to the end of the destination &lt;paramref name="array"/&gt;.&lt;/para&gt;
		/// &lt;/exception&gt;
		/// &lt;exception cref="InvalidCastException"&gt;
		///		The type of the source &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; cannot be cast automatically to the type
		///		of the destination &lt;paramref name="array"/&gt;.
		/// &lt;/exception&gt;
		public virtual void CopyTo(Array array, int arrayIndex)
		{
			if (array == null)
				throw new ArgumentNullException("array", "The destination array cannot be null.");
			if (arrayIndex &lt; 0)
				throw new ArgumentOutOfRangeException("arrayIndex", "Destination index cannot be less than zero.");
			if (array.Rank != 1)
				throw new ArgumentException("Multidimensional arrays are not supported.", "array");
			if (arrayIndex &gt;= array.Length)
				throw new ArgumentException("Destination index cannot be greater than the size of the destination array.", "arrayIndex");
			if (count &gt; (array.Length - arrayIndex))
				throw new ArgumentException("Not enough available space in the destination array.");

			for (int i=0; i &lt; count; i++)
			{
				DictionaryEntry entry = new DictionaryEntry(keys[i], values[i]);
				array.SetValue(entry, arrayIndex + i);
			}
		}

		/// &lt;summary&gt;
		///		Gets the value at the specified index of the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.
		/// &lt;/summary&gt;
		/// &lt;param name="index"&gt;The zero-based index of the value to get.&lt;/param&gt;
		/// &lt;returns&gt;The value at the specified index of the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.&lt;/returns&gt;
		/// &lt;exception cref="ArgumentOutOfRangeException"&gt;
		///		&lt;paramref name="index"/&gt; is outside the range of valid indices for the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.
		/// &lt;/exception&gt;
		public virtual <xsl:value-of select='itemType' /> GetByIndex(int index)
		{
			if (index &lt; 0 || index &gt;= count)
				throw new ArgumentOutOfRangeException("index", index, "The index is outside the range of valid indices.");

			return values[index];
		}

		/// &lt;summary&gt;
		///		Returns an &lt;see cref="IEnumerator"/&gt; that can iterate through the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.
		/// &lt;/summary&gt;
		IEnumerator IEnumerable.GetEnumerator()
		{
			return new SortedListEnumerator(this, 0, count, SortedListEnumerator.DictEntry);
		}

		/// &lt;summary&gt;
		///		Returns an &lt;see cref="IDictionaryEnumerator"/&gt; that can iterate through the
		///		&lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.		
		/// &lt;/summary&gt;
		/// &lt;returns&gt;An &lt;see cref="IDictionaryEnumerator"/&gt; for the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.&lt;/returns&gt;
		public virtual IDictionaryEnumerator GetEnumerator()
		{
			return new SortedListEnumerator(this, 0, count, SortedListEnumerator.DictEntry);
		}

		/// &lt;summary&gt;
		///		Gets the key at the specified index of the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.
		/// &lt;/summary&gt;
		/// &lt;param name="index"&gt;The zero-based index of the key to get.&lt;/param&gt;
		/// &lt;returns&gt;The key at the specified index of the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.&lt;/returns&gt;
		/// &lt;exception cref="ArgumentOutOfRangeException"&gt;
		///		&lt;paramref name="index"/&gt; is outside the range of valid indices for the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.
		/// &lt;/exception&gt;
		public virtual <xsl:value-of select='keyType' /> GetKey(int index)
		{
			if (index &lt; 0 || index &gt;= count)
				throw new ArgumentOutOfRangeException("index", index, "The index is outside the range of valid indices.");

			return keys[index];
		}

		/// &lt;summary&gt;
		///		Gets the keys in the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.
		/// &lt;/summary&gt;
		/// &lt;returns&gt;An &lt;see cref="IList"/&gt; containing the keys in the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.&lt;/returns&gt;
		/// &lt;remarks&gt;
		///		&lt;para&gt;The returned &lt;see cref="IList"/&gt; is a read-only view of the keys in the 
		///		&lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;. Modifications made to the underlying &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;
		///		are immediately reflected in the &lt;b&gt;IList&lt;/b&gt;.&lt;/para&gt;
		///		&lt;para&gt;The elements of the &lt;b&gt;IList&lt;/b&gt; are sorted in the same order as the keys
		///		of the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.&lt;/para&gt;
		///		&lt;para&gt;Similar to &lt;see cref="<xsl:value-of select='collectionName'/>.Keys"/&gt;, but returns an &lt;b&gt;IList&lt;/b&gt;
		///		instead of an &lt;see cref="ICollection"/&gt;.&lt;/para&gt;
		/// &lt;/remarks&gt;
		public virtual IList GetKeyList()
		{
			if (keyList == null)
				keyList = new KeyList(this);

			return keyList;
		}

		/// &lt;summary&gt;
		///		Gets the values in the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.
		/// &lt;/summary&gt;
		/// &lt;returns&gt;An &lt;see cref="IList"/&gt; containing the values in the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.&lt;/returns&gt;
		/// &lt;remarks&gt;
		///		&lt;para&gt;The returned &lt;see cref="IList"/&gt; is a read-only view of the values in the 
		///		&lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;. Modifications made to the underlying &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;
		///		are immediately reflected in the &lt;b&gt;IList&lt;/b&gt;.&lt;/para&gt;
		///		&lt;para&gt;The elements of the &lt;b&gt;IList&lt;/b&gt; are sorted in the same order as the values
		///		of the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.&lt;/para&gt;
		///		&lt;para&gt;Similar to &lt;see cref="<xsl:value-of select='collectionName'/>.Values"/&gt;, but returns an &lt;b&gt;IList&lt;/b&gt;
		///		instead of an &lt;see cref="ICollection"/&gt;.&lt;/para&gt;
		/// &lt;/remarks&gt;
		public virtual IList GetValueList()
		{
			if (valueList == null)
				valueList = new ValueList(this);

			return valueList;
		}

		/// &lt;summary&gt;
		///		Returns the zero-based index of the specified key in the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.
		/// &lt;/summary&gt;
		/// &lt;param name="key"&gt;The key to locate in the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.&lt;/param&gt;
		/// &lt;returns&gt;
		///		The zero-based index of &lt;paramref name="key"/&gt;, if &lt;paramref name="key"/&gt; is found in
		///		the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;; otherwise, -1.
		/// &lt;/returns&gt;
		/// &lt;exception cref="ArgumentNullException"&gt;&lt;paramref name="key"/&gt; is a null reference.&lt;/exception&gt;
		/// &lt;exception cref="InvalidOperationException"&gt;The comparer throws an exception.&lt;/exception&gt;
		/// &lt;remarks&gt;
		///		&lt;para&gt;The elements of a &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; are sorted by the keys either 
		///		according to a specific &lt;see cref="IComparer"/&gt; implementation specified when the 
		///		&lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; is created or according to the &lt;see cref="IComparable"/&gt; 
		///		implementation provided by the keys themselves.&lt;/para&gt;
		///		&lt;para&gt;The index sequence is based on the sort sequence. When an element is added, 
		///		it is inserted into &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; in the correct sort order, and the 
		///		indexing adjusts accordingly. When an element removed, the indexing also adjusts accordingly.
		///		Therefore, the index of a specific key-and-value pair might change as elements are added or 
		///		removed from the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.&lt;/para&gt;
		///		&lt;para&gt;This method uses a binary search algorithm; therefore, the average execution time is 
		///		proportional to Log2(&lt;i&gt;n&lt;/i&gt;), where &lt;i&gt;n&lt;/i&gt; is &lt;see cref="<xsl:value-of select='collectionName'/>.Count"/&gt;.&lt;/para&gt;
		/// &lt;/remarks&gt;
		public virtual int IndexOfKey(<xsl:value-of select='keyType' /> key)
		{
			if (Object.ReferenceEquals(key, null)) // avoids compiler error for null check on value type
				throw new ArgumentNullException("key", "The key cannot be null.");

			int index = Array.BinarySearch(keys, 0, count, key, comparer);

			return (index &gt;= 0 ? index : -1);
		}

		/// &lt;summary&gt;
		///		Returns the zero-based index of the first occurrence of the specified value in
		///		the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.
		/// &lt;/summary&gt;
		/// &lt;param name="value"&gt;The value to locate in the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.&lt;/param&gt;
		/// &lt;returns&gt;
		///		The zero-based index of &lt;paramref name="value"/&gt;, if &lt;paramref name="value"/&gt; is found in
		///		the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;; otherwise, -1.
		/// &lt;/returns&gt;
		/// &lt;remarks&gt;
		///		&lt;para&gt;The index sequence is based on the sort sequence. When an element is added, 
		///		it is inserted into &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; in the correct sort order, and 
		///		the indexing adjusts accordingly. When an element removed, the indexing also adjusts 
		///		accordingly. Therefore, the index of a specific key-and-value pair might change as 
		///		elements are added or removed from the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.&lt;/para&gt;
		///		&lt;para&gt;The values of the elements of the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; are compared to the 
		///		specified value using the Equals method.&lt;/para&gt;
		///		&lt;para&gt;This method uses a linear search; therefore, the average execution time is 
		///		proportional to &lt;see cref="<xsl:value-of select='collectionName'/>.Count"/&gt;.&lt;/para&gt;
		/// &lt;/remarks&gt;
		public virtual int IndexOfValue(<xsl:value-of select='itemType' /> value)
		{
			return Array.IndexOf(values, value, 0, count);
		}

		/// &lt;summary&gt;
		///		Removes the element with the specified key from the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.
		/// &lt;/summary&gt;
		/// &lt;param name="key"&gt;The key of the element to remove.&lt;/param&gt;
		/// &lt;exception cref="ArgumentException"&gt;
		///		The &lt;paramref name="key"/&gt; is not of a type supported by the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.
		/// &lt;/exception&gt;
		/// &lt;exception cref="ArgumentNullException"&gt;&lt;paramref name="key"/&gt; is a null reference.&lt;/exception&gt;
		/// &lt;exception cref="NotSupportedException"&gt;
		///		&lt;para&gt;The &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; is read-only.&lt;/para&gt;
		///		&lt;para&gt;-or-&lt;/para&gt;
		///		&lt;para&gt;The &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt; has a fixed size.&lt;/para&gt;
		/// &lt;/exception&gt;
		void IDictionary.Remove(object key)
		{
			if (key == null)
				throw new ArgumentNullException("key", "The key cannot be null.");

			if (!(key is <xsl:value-of select='keyType' />))
				throw new ArgumentException("The key must be of type: " + typeof(<xsl:value-of select='keyType' />).FullName, "key");

			Remove((<xsl:value-of select='keyType' />)key);
		}

		/// &lt;summary&gt;
		///		Removes the element with the specified key from the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.
		/// &lt;/summary&gt;
		/// &lt;param name="key"&gt;The key of the element to remove.&lt;/param&gt;
		/// &lt;exception cref="ArgumentNullException"&gt;&lt;paramref name="key"/&gt; is a null reference.&lt;/exception&gt;
		/// &lt;exception cref="NotSupportedException"&gt;
		///		&lt;para&gt;The &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; is read-only.&lt;/para&gt;
		///		&lt;para&gt;-or-&lt;/para&gt;
		///		&lt;para&gt;The &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt; has a fixed size.&lt;/para&gt;
		/// &lt;/exception&gt;
		/// &lt;remarks&gt;
		///		If the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; does not contain an element with the specified key,
		///		the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt; remains unchanged. No exception is thrown.
		/// &lt;/remarks&gt;
		public virtual void Remove(<xsl:value-of select='keyType' /> key)
		{
			if (Object.ReferenceEquals(key, null)) // avoids compiler error for null check on value type
				throw new ArgumentNullException("key", "The key cannot be null.");

			int index = IndexOfKey(key);
			if (index &gt;= 0)
				RemoveAt(index);
		}

		/// &lt;summary&gt;
		///		Removes the element at the specified index of the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.
		/// &lt;/summary&gt;
		/// &lt;param name="index"&gt;The zero-based index of the element to remove.&lt;/param&gt;
		/// &lt;exception cref="ArgumentOutOfRangeException"&gt;
		///		&lt;paramref name="index"/&gt; is outside the range of valid indices for the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.
		///	&lt;/exception&gt;
		/// &lt;exception cref="NotSupportedException"&gt;
		///		&lt;para&gt;The &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; is read-only.&lt;/para&gt;
		///		&lt;para&gt;-or-&lt;/para&gt;
		///		&lt;para&gt;The &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt; has a fixed size.&lt;/para&gt;
		/// &lt;/exception&gt;
		/// &lt;remarks&gt;
		///		&lt;para&gt;The index sequence is based on the sort sequence. When an element is added, 
		///		it is inserted into &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; in the correct sort order, and 
		///		the indexing adjusts accordingly. When an element removed, the indexing also adjusts 
		///		accordingly. Therefore, the index of a specific key-and-value pair might change as 
		///		elements are added or removed from the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.&lt;/para&gt;
		///		&lt;para&gt;In collections of contiguous elements, such as lists, the elements that
		///		follow the removed element move up to occupy the vacated spot. If the collection is
		///		indexed, the indices of the elements that are moved are also updated.&lt;/para&gt;
		/// &lt;/remarks&gt;
		public virtual void RemoveAt(int index)
		{
			if (index &lt; 0 || index &gt;= count)
				throw new ArgumentOutOfRangeException("index", index, "The index is outside the range of valid indices.");

			count--;
			if (index &lt; count)
			{
				Array.Copy(keys, index + 1, keys, index, count - index);
				Array.Copy(values, index + 1, values, index, count - index);
			}

			// We can't set the deleted entries equal to null, because they might be value types.
			// Instead, we'll create empty single-element arrays of the right type and copy them 
			// over the entries we want to erase.
			<xsl:value-of select='keyType' />[] tempKey = new <xsl:value-of select='keyType' />[1];
			<xsl:value-of select='itemType' />[] tempVal = new <xsl:value-of select='itemType' />[1];
			Array.Copy(tempKey, 0, keys, count, 1);
			Array.Copy(tempVal, 0, values, count, 1);

			version++;
		}

		/// &lt;summary&gt;
		///		Replaces the value at a specific index in the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.
		/// &lt;/summary&gt;
		/// &lt;param name="index"&gt;The zero-based index at which to save &lt;paramref name="value"/&gt;.&lt;/param&gt;
		/// &lt;param name="value"&gt;The &lt;see cref="<xsl:value-of select='itemType' />"/&gt; to save into the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.&lt;/param&gt;
		/// &lt;exception cref="ArgumentOutOfRangeException"&gt;
		///		&lt;paramref name="index"/&gt; is outside the range of valid indices for the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.
		/// &lt;/exception&gt;
		/// &lt;remarks&gt;
		///		&lt;para&gt;The index sequence is based on the sort sequence. When an element is added, 
		///		it is inserted into &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; in the correct sort order, and 
		///		the indexing adjusts accordingly. When an element removed, the indexing also adjusts 
		///		accordingly. Therefore, the index of a specific key-and-value pair might change as 
		///		elements are added or removed from the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.&lt;/para&gt;
		/// &lt;/remarks&gt;
		public virtual void SetByIndex(int index, <xsl:value-of select='itemType' /> value)
		{
			if (index &lt; 0 || index &gt;= count)
				throw new ArgumentOutOfRangeException("index", index, "The index is outside the range of valid indices.");

			values[index] = value;
			version++;
		}

		/// &lt;summary&gt;
		///		Returns a synchronized (thread-safe) wrapper for the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.
		/// &lt;/summary&gt;
		/// &lt;param name="list"&gt;The &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; to synchronize.&lt;/param&gt;
		/// &lt;returns&gt;A synchronized (thread-safe) wrapper for the &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt;.&lt;/returns&gt;
		/// &lt;exception cref="ArgumentNullException"&gt;&lt;paramref name="list"/&gt; is a null reference.&lt;/exception&gt;
		public static <xsl:value-of select='collectionName'/> Synchronized(<xsl:value-of select='collectionName'/> list)
		{
			if (list == null)
				throw new ArgumentNullException("list", "The list cannot be null.");

			return new SyncSortedList(list);
		}
		
		/// &lt;summary&gt;
		///		Sets the capacity to the actual number of elements in the &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt;.
		/// &lt;/summary&gt;
		/// &lt;exception cref="NotSupportedException"&gt;
		///		&lt;para&gt;The &lt;see cref="<xsl:value-of select='collectionName'/>"/&gt; is read-only.&lt;/para&gt;
		///		&lt;para&gt;-or-&lt;/para&gt;
		///		&lt;para&gt;The &lt;b&gt;<xsl:value-of select='collectionName'/>&lt;/b&gt; has a fixed size.&lt;/para&gt;
		/// &lt;/exception&gt;
		public virtual void TrimToSize()
		{
			this.Capacity = count;
		}
		#endregion

		#region Private Methods
		private void Insert(int index, <xsl:value-of select='keyType' /> key, <xsl:value-of select='itemType' /> value)
		{
			if (count == keys.Length)
				EnsureCapacity(count + 1);

			if (index &lt; count)
			{
				Array.Copy(keys, index, keys, index + 1, count - index);
				Array.Copy(values, index, values, index + 1, count - index);
			}

			keys[index] = key;
			values[index] = value;
			count++;
			version++;
		}

		private void EnsureCapacity(int min)
		{
			int newCapacity = ((keys.Length == 0) ? DEFAULT_CAPACITY : keys.Length * 2);
			if (newCapacity &lt; min)
				newCapacity = min;

			this.Capacity = newCapacity;
		}
		#endregion

		#region Nested Class: SyncSortedList
		[Serializable]
		private class SyncSortedList : <xsl:value-of select='collectionName'/>, IDictionary
		{
			private <xsl:value-of select='collectionName'/> list;
			private object root;

			internal SyncSortedList(<xsl:value-of select='collectionName'/> list)
			{
				this.list = list;
				this.root = list.SyncRoot;
			}

			public override int Capacity
			{
				get
				{
					lock (root)
						return list.Capacity;
				}
			}

			public override int Count
			{
				get
				{
					lock (root)
						return list.Count;
				}
			}

			public override bool IsFixedSize
			{
				get
				{
					return list.IsFixedSize;
				}
			}

			public override bool IsReadOnly
			{
				get
				{
					return list.IsReadOnly;
				}
			}

			public override bool IsSynchronized
			{
				get
				{
					return true;
				}
			}

			public override object SyncRoot
			{
				get
				{
					return root;
				}
			}

			object IDictionary.this[object key]
			{
				get
				{
					lock (root)
						return ((IDictionary)list)[key];
				}
				set
				{
					lock (root)
						((IDictionary)list)[key] = value;
				}
			}

			public override <xsl:value-of select='itemType' /> this[<xsl:value-of select='keyType' /> key]
			{
				get
				{
					lock (root)
						return list[key];
				}
				set
				{
					lock (root)
						list[key] = value;
				}
			}

			void IDictionary.Add(object key, object value)
			{
				lock (root)
					((IDictionary)list).Add(key, value);
			}

			public override void Add(<xsl:value-of select='keyType' /> key, <xsl:value-of select='itemType' /> value)
			{
				lock (root)
					list.Add(key, value);
			}

			public override void Clear()
			{
				lock (root)
					list.Clear();
			}

			public override object Clone()
			{
				lock (root)
					return list.Clone();
			}

			bool IDictionary.Contains(object key)
			{
				lock (root)
					return ((IDictionary)list).Contains(key);
			}

			public override bool Contains(<xsl:value-of select='keyType' /> key)
			{
				lock (root)
					return list.Contains(key);
			}

			public override bool ContainsKey(<xsl:value-of select='keyType' /> key)
			{
				lock (root)
					return list.ContainsKey(key);
			}

			public override bool ContainsValue(<xsl:value-of select='itemType' /> value)
			{
				lock (root)
					return list.ContainsValue(value);
			}

			public override void CopyTo(Array array, int index)
			{
				lock (root)
					list.CopyTo(array, index);
			}

			public override <xsl:value-of select='itemType' /> GetByIndex(int index)
			{
				lock (root)
					return list.GetByIndex(index);
			}

			public override IDictionaryEnumerator GetEnumerator()
			{
				lock (root)
					return list.GetEnumerator();
			}

			public override <xsl:value-of select='keyType' /> GetKey(int index)
			{
				lock (root)
					return list.GetKey(index);
			}

			public override IList GetKeyList()
			{
				lock (root)
					return list.GetKeyList();
			}

			public override IList GetValueList()
			{
				lock (root)
					return list.GetValueList();
			}

			public override int IndexOfKey(<xsl:value-of select='keyType' /> key)
			{
				lock (root)
					return list.IndexOfKey(key);
			}

			public override int IndexOfValue(<xsl:value-of select='itemType' /> value)
			{
				lock (root)
					return list.IndexOfValue(value);
			}

			void IDictionary.Remove(object key)
			{
				lock (root)
					((IDictionary)list).Remove(key);
			}

			public override void Remove(<xsl:value-of select='keyType' /> key)
			{
				lock (root)
					list.Remove(key);
			}

			public override void RemoveAt(int index)
			{
				lock (root)
					list.RemoveAt(index);
			}

			public override void SetByIndex(int index, <xsl:value-of select='itemType' /> value)
			{
				lock (root)
					list.SetByIndex(index, value);
			}

			public override void TrimToSize()
			{
				lock (root)
					list.TrimToSize();
			}
		}
		#endregion

		#region Nested Class: SortedListEnumerator
		[Serializable]
		private class SortedListEnumerator : IDictionaryEnumerator, ICloneable
		{
			private <xsl:value-of select='collectionName'/> list;
			private <xsl:value-of select='keyType' /> key;
			private <xsl:value-of select='itemType' /> value;
			private int index;
			private int startIndex;
			private int endIndex;
			private int version;
			private bool currentValid;
			private int returnType;

			internal const int Keys = 1;
			internal const int Values = 2;
			internal const int DictEntry = 3;

			internal SortedListEnumerator(<xsl:value-of select='collectionName'/> list, int index, int count, int returnType)
			{
				this.list = list;
				this.index = index;
				this.startIndex = index;
				this.endIndex = index + count;
				this.version = list.version;
				this.returnType = returnType;
				this.currentValid = false;
			}

			public object Clone()
			{
				return this.MemberwiseClone();
			}

			object IDictionaryEnumerator.Key
			{
				get
				{
					CheckState();
					return key;
				}
			}

			public virtual <xsl:value-of select='keyType' /> Key
			{
				get
				{
					CheckState();
					return key;
				}
			}

			public virtual DictionaryEntry Entry
			{
				get
				{
					CheckState();
					return new DictionaryEntry(key, value);
				}
			}

			public virtual object Current
			{
				get
				{
					CheckState();

					switch (returnType)
					{
						case Keys:
							return key;
						case Values:
							return value;
						case DictEntry:
						default:
							return new DictionaryEntry(key, value);
					}
				}
			}

			object IDictionaryEnumerator.Value
			{
				get
				{
					CheckState();
					return value;
				}
			}

			public virtual <xsl:value-of select='itemType' /> Value
			{
				get
				{
					CheckState();
					return value;
				}
			}

			public virtual bool MoveNext()
			{
				if (version != list.version)
					throw new InvalidOperationException("The collection was modified - enumeration cannot continue.");
				
				if (index &lt; endIndex)
				{
					key = list.keys[index];
					value = list.values[index];
					index++;
					currentValid = true;
					return true;
				}

				// We can't set the entries equal to null, because they might be value types.
				// Instead, we'll create empty single-element arrays of the right type and copy them 
				// over the entries we want to erase.
				<xsl:value-of select='keyType' />[] tempKey = new <xsl:value-of select='keyType' />[1];
				<xsl:value-of select='itemType' />[] tempVal = new <xsl:value-of select='itemType' />[1];
				key = tempKey[0];
				value = tempVal[0];
				currentValid = false;
				return false;
			}

			public virtual void Reset()
			{
				if (version != list.version)
					throw new InvalidOperationException("The collection was modified - enumeration cannot continue.");

				// We can't set the entries equal to null, because they might be value types.
				// Instead, we'll create empty single-element arrays of the right type and copy them 
				// over the entries we want to erase.
				<xsl:value-of select='keyType' />[] tempKey = new <xsl:value-of select='keyType' />[1];
				<xsl:value-of select='itemType' />[] tempVal = new <xsl:value-of select='itemType' />[1];
				key = tempKey[0];
				value = tempVal[0];
				currentValid = false;
				index = startIndex;
			}

			private void CheckState()
			{
				if (version != list.version)
					throw new InvalidOperationException("The collection was modified - enumeration cannot continue.");
				if (!currentValid)
					throw new InvalidOperationException("Enumeration either has not started or has already finished.");
			}
		}
		#endregion

		#region Nested Class: KeyList
		[Serializable]
		private class KeyList : IList
		{
			private <xsl:value-of select='collectionName'/> list;

			internal KeyList(<xsl:value-of select='collectionName'/> list)
			{
				this.list = list;
			}

			public virtual int Count
			{
				get
				{
					return list.Count;
				}
			}

			public virtual bool IsReadOnly
			{
				get { return true; }
			}

			public virtual bool IsFixedSize
			{
				get { return true; }
			}

			public virtual bool IsSynchronized
			{
				get { return list.IsSynchronized; }
			}

			public virtual object SyncRoot
			{
				get { return list.SyncRoot; }
			}

			public virtual int Add(object key)
			{
				throw new NotSupportedException("Cannot add to a read-only list.");
			}

			public virtual void Clear()
			{
				throw new NotSupportedException("Cannot clear a read-only list.");
			}

			bool IList.Contains(object key)
			{
				return ((IDictionary)list).Contains(key);
			}

			public virtual bool Contains(<xsl:value-of select='keyType' /> key)
			{
				return list.Contains(key);
			}

			public virtual void CopyTo(Array array, int index)
			{
				if (array != null &amp;&amp; array.Rank != 1)
					throw new ArgumentException("Multidimensional arrays are not supported.", "array");

				Array.Copy(list.keys, 0, array, index, list.Count);
			}

			public virtual void Insert(int index, object value)
			{
				throw new NotSupportedException("Cannot insert into a read-only list.");
			}

			object IList.this[int index]
			{
				get
				{
					return list.GetKey(index);
				}
				set
				{
					throw new NotSupportedException("Cannot modify a read-only list.");
				}
			}

			public virtual <xsl:value-of select='keyType' /> this[int index]
			{
				get
				{
					return list.GetKey(index);
				}
				set
				{
					throw new NotSupportedException("Cannot modify a read-only list.");
				}
			}

			public virtual IEnumerator GetEnumerator()
			{
				return new SortedListEnumerator(list, 0, list.Count, SortedListEnumerator.Keys);
			}

			int IList.IndexOf(object key)
			{
				if (!(key is <xsl:value-of select='keyType' />))
					throw new ArgumentException("The key must be of type: " + typeof(<xsl:value-of select='keyType' />).FullName, "key");

				return list.IndexOfKey((<xsl:value-of select='keyType' />)key);
			}

			public virtual int IndexOf(<xsl:value-of select='keyType' /> key)
			{
				return list.IndexOfKey(key);
			}

			public virtual void Remove(object key)
			{
				throw new NotSupportedException("Cannot modify a read-only list.");
			}

			public virtual void RemoveAt(int index)
			{
				throw new NotSupportedException("Cannot modify a read-only list.");
			}
		}
		#endregion

		#region Nested Class: ValueList
		[Serializable]
		private class ValueList : IList
		{
			private <xsl:value-of select='collectionName'/> list;

			internal ValueList(<xsl:value-of select='collectionName'/> list)
			{
				this.list = list;
			}

			public virtual int Count
			{
				get
				{
					return list.Count;
				}
			}

			public virtual bool IsReadOnly
			{
				get { return true; }
			}

			public virtual bool IsFixedSize
			{
				get { return true; }
			}

			public virtual bool IsSynchronized
			{
				get { return list.IsSynchronized; }
			}

			public virtual object SyncRoot
			{
				get { return list.SyncRoot; }
			}

			public virtual int Add(object key)
			{
				throw new NotSupportedException("Cannot add to a read-only list.");
			}

			public virtual void Clear()
			{
				throw new NotSupportedException("Cannot clear a read-only list.");
			}

			bool IList.Contains(object value)
			{
				if (!(value is <xsl:value-of select='itemType' />))
					throw new ArgumentException("The value must be of type: " + typeof(<xsl:value-of select='itemType' />).FullName, "value");

				return list.ContainsValue((<xsl:value-of select='itemType' />)value);
			}

			public virtual bool Contains(<xsl:value-of select='itemType' /> value)
			{
				return list.ContainsValue(value);
			}

			public virtual void CopyTo(Array array, int index)
			{
				if (array != null &amp;&amp; array.Rank != 1)
					throw new ArgumentException("Multidimensional arrays are not supported.", "array");

				Array.Copy(list.values, 0, array, index, list.Count);
			}

			public virtual void Insert(int index, object value)
			{
				throw new NotSupportedException("Cannot insert into a read-only list.");
			}

			object IList.this[int index]
			{
				get
				{
					return list.GetByIndex(index);
				}
				set
				{
					if (!(value is <xsl:value-of select='itemType' />))
						throw new ArgumentException("The value must be of type: " + typeof(<xsl:value-of select='itemType' />).FullName, "value");

					list.SetByIndex(index, (<xsl:value-of select='itemType' />)value);
				}
			}

			public virtual <xsl:value-of select='itemType' /> this[int index]
			{
				get
				{
					return list.GetByIndex(index);
				}
				set
				{
					list.SetByIndex(index, value);
				}
			}

			public virtual IEnumerator GetEnumerator()
			{
				return new SortedListEnumerator(list, 0, list.Count, SortedListEnumerator.Values);
			}

			int IList.IndexOf(object value)
			{
				if (!(value is <xsl:value-of select='itemType' />))
					throw new ArgumentException("The value must be of type: " + typeof(<xsl:value-of select='itemType' />).FullName, "value");

				return list.IndexOfValue((<xsl:value-of select='itemType' />)value);
			}

			public virtual int IndexOf(<xsl:value-of select='itemType' /> value)
			{
				return list.IndexOfValue(value);
			}

			public virtual void Remove(object key)
			{
				throw new NotSupportedException("Cannot modify a read-only list.");
			}

			public virtual void RemoveAt(int index)
			{
				throw new NotSupportedException("Cannot modify a read-only list.");
			}
		}
		#endregion
	}
<xsl:if test="collectionNamespace">
	<xsl:if test="string-length(collectionNamespace) > 0">
}
	</xsl:if>
</xsl:if>

  </xsl:template>

</xsl:transform>