<!-- CSVector.xslt -->
<!-- This code leveraged from Shawn Van Ness [shawnv@arithex.com] -->
<!-- http://www.develop.com/software/GenXdownloaddetail.asp?vWizId=281 -->
<!-- Updated by Matthias Hess [matthias.hess@bluewin.ch] to fix XSLT "garbage" problem -->
<!-- Updated by Tor-Erik Hagen [tor-erik@web-amp.com] to make it possible not to specify namespace -->
<!-- Updated by Justin Rudd [Justin_Rudd@Trimble.com] to make synchronized access more efficient -->
<!-- Updated by James Berry [james@adastra.co.uk] to fix a data-binding problem with ICollection.CopyTo -->
<xsl:transform version='1.0' xmlns:xsl='http://www.w3.org/1999/XSL/Transform'>
  <xsl:output method='text'/>
  
  <xsl:template match="/">
	<xsl:apply-templates select='/typeSafeCollections/typeSafeCollection[templateKind="Vector"]'/>
  </xsl:template>

  <xsl:template match='/typeSafeCollections/typeSafeCollection[templateKind="Vector"]'>

<xsl:if test="collectionNamespace">
	<xsl:if test="string-length(collectionNamespace) > 0">
namespace <xsl:value-of select='collectionNamespace' />
{
	</xsl:if>
</xsl:if>

	/// &lt;summary&gt;
	///		A strongly-typed collection of &lt;see cref="<xsl:value-of select="itemType"/>"/&gt; objects.
	/// &lt;/summary&gt;
	[Serializable]
<xsl:choose>
  <xsl:when test="accessibility"><xsl:value-of select='accessibility' /></xsl:when>
	<xsl:otherwise>public </xsl:otherwise>
</xsl:choose>
  class <xsl:value-of select='collectionName' /> : ICollection, IList, IEnumerable, ICloneable
	{
        #region Interfaces
	    /// &lt;summary&gt;
	    ///		Supports type-safe iteration over a &lt;see cref="<xsl:value-of select="collectionName"/>"/&gt;.
	    /// &lt;/summary&gt;
        public interface I<xsl:value-of select='collectionName'/>Enumerator
        {
		    /// &lt;summary&gt;
		    ///		Gets the current element in the collection.
		    /// &lt;/summary&gt;
            <xsl:value-of select='itemType' /> Current {get;}

		    /// &lt;summary&gt;
		    ///		Advances the enumerator to the next element in the collection.
		    /// &lt;/summary&gt;
		    /// &lt;exception cref="InvalidOperationException"&gt;
		    ///		The collection was modified after the enumerator was created.
		    /// &lt;/exception&gt;
		    /// &lt;returns&gt;
		    ///		&lt;c&gt;true&lt;/c&gt; if the enumerator was successfully advanced to the next element; 
		    ///		&lt;c&gt;false&lt;/c&gt; if the enumerator has passed the end of the collection.
		    /// &lt;/returns&gt;
            bool MoveNext();

		    /// &lt;summary&gt;
		    ///		Sets the enumerator to its initial position, before the first element in the collection.
		    /// &lt;/summary&gt;
            void Reset();
        }
        #endregion

		private const int DEFAULT_CAPACITY = 16;

		#region Implementation (data)
		private <xsl:value-of select='itemType' />[] m_array;
		private int m_count = 0;
		[NonSerialized]
		private int m_version = 0;
		#endregion
	
        #region Static Wrappers
		/// &lt;summary&gt;
		///		Creates a synchronized (thread-safe) wrapper for a 
		///     &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt; instance.
		/// &lt;/summary&gt;
		/// &lt;returns&gt;
		///     An &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt; wrapper that is synchronized (thread-safe).
		/// &lt;/returns&gt;
        public static <xsl:value-of select='collectionName'/> Synchronized(<xsl:value-of select='collectionName'/> list)
        {
            if(list==null)
                throw new ArgumentNullException("list");
            return new Sync<xsl:value-of select='collectionName'/>(list);
        }
        
		/// &lt;summary&gt;
		///		Creates a read-only wrapper for a 
		///     &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt; instance.
		/// &lt;/summary&gt;
		/// &lt;returns&gt;
		///     An &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt; wrapper that is read-only.
		/// &lt;/returns&gt;
        public static <xsl:value-of select='collectionName'/> ReadOnly(<xsl:value-of select='collectionName'/> list)
        {
            if(list==null)
                throw new ArgumentNullException("list");
            return new ReadOnly<xsl:value-of select='collectionName'/>(list);
        }
        #endregion

	    #region Construction
		/// &lt;summary&gt;
		///		Initializes a new instance of the &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt; class
		///		that is empty and has the default initial capacity.
		/// &lt;/summary&gt;
		public <xsl:value-of select='collectionName' />()
		{
			m_array = new <xsl:value-of select='itemType' />[DEFAULT_CAPACITY];
		}
		
		/// &lt;summary&gt;
		///		Initializes a new instance of the &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt; class
		///		that has the specified initial capacity.
		/// &lt;/summary&gt;
		/// &lt;param name="capacity"&gt;
		///		The number of elements that the new &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt; is initially capable of storing.
		///	&lt;/param&gt;
		public <xsl:value-of select='collectionName' />(int capacity)
		{
			m_array = new <xsl:value-of select='itemType' />[capacity];
		}

		/// &lt;summary&gt;
		///		Initializes a new instance of the &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt; class
		///		that contains elements copied from the specified &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt;.
		/// &lt;/summary&gt;
		/// &lt;param name="c"&gt;The &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt; whose elements are copied to the new collection.&lt;/param&gt;
		public <xsl:value-of select='collectionName' />(<xsl:value-of select='collectionName' /> c)
		{
			m_array = new <xsl:value-of select='itemType' />[c.Count];
			AddRange(c);
		}

		/// &lt;summary&gt;
		///		Initializes a new instance of the &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt; class
		///		that contains elements copied from the specified &lt;see cref="<xsl:value-of select="itemType"/>"/&gt; array.
		/// &lt;/summary&gt;
		/// &lt;param name="a"&gt;The &lt;see cref="<xsl:value-of select="itemType"/>"/&gt; array whose elements are copied to the new list.&lt;/param&gt;
		public <xsl:value-of select='collectionName' />(<xsl:value-of select='itemType' />[] a)
		{
			m_array = new <xsl:value-of select='itemType' />[a.Length];
			AddRange(a);
		}
		
        protected enum Tag {
            Default
        }

        protected <xsl:value-of select='collectionName'/>(Tag t)
        {
            m_array = null;
        }
		#endregion
		
		#region Operations (type-safe ICollection)
		/// &lt;summary&gt;
		///		Gets the number of elements actually contained in the &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt;.
		/// &lt;/summary&gt;
		public virtual int Count
		{
			get { return m_count; }
		}

		/// &lt;summary&gt;
		///		Copies the entire &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt; to a one-dimensional
		///		&lt;see cref="<xsl:value-of select="itemType"/>"/&gt; array.
		/// &lt;/summary&gt;
		/// &lt;param name="array"&gt;The one-dimensional &lt;see cref="<xsl:value-of select="itemType"/>"/&gt; array to copy to.&lt;/param&gt;
		public virtual void CopyTo(<xsl:value-of select='itemType' />[] array)
		{
			this.CopyTo(array, 0);
		}

		/// &lt;summary&gt;
		///		Copies the entire &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt; to a one-dimensional
		///		&lt;see cref="<xsl:value-of select="itemType"/>"/&gt; array, starting at the specified index of the target array.
		/// &lt;/summary&gt;
		/// &lt;param name="array"&gt;The one-dimensional &lt;see cref="<xsl:value-of select="itemType"/>"/&gt; array to copy to.&lt;/param&gt;
		/// &lt;param name="start"&gt;The zero-based index in &lt;paramref name="array"/&gt; at which copying begins.&lt;/param&gt;
		public virtual void CopyTo(<xsl:value-of select='itemType' />[] array, int start)
		{
			if (m_count &gt; array.GetUpperBound(0) + 1 - start)
				throw new System.ArgumentException("Destination array was not long enough.");
			
			Array.Copy(m_array, 0, array, start, m_count); 
		}

		/// &lt;summary&gt;
		///		Gets a value indicating whether access to the collection is synchronized (thread-safe).
		/// &lt;/summary&gt;
		/// &lt;returns&gt;true if access to the ICollection is synchronized (thread-safe); otherwise, false.&lt;/returns&gt;
        public virtual bool IsSynchronized
        {
            get { return m_array.IsSynchronized; }
        }

        /// &lt;summary&gt;
		///		Gets an object that can be used to synchronize access to the collection.
		/// &lt;/summary&gt;
        public virtual object SyncRoot
        {
            get { return m_array.SyncRoot; }
        }
		#endregion
		
		#region Operations (type-safe IList)
		/// &lt;summary&gt;
		///		Gets or sets the &lt;see cref="<xsl:value-of select="itemType"/>"/&gt; at the specified index.
		/// &lt;/summary&gt;
		/// &lt;param name="index"&gt;The zero-based index of the element to get or set.&lt;/param&gt;
		/// &lt;exception cref="ArgumentOutOfRangeException"&gt;
		///		&lt;para&gt;&lt;paramref name="index"/&gt; is less than zero&lt;/para&gt;
		///		&lt;para&gt;-or-&lt;/para&gt;
		///		&lt;para&gt;&lt;paramref name="index"/&gt; is equal to or greater than &lt;see cref="<xsl:value-of select="collectionName"/>.Count"/&gt;.&lt;/para&gt;
		/// &lt;/exception&gt;
		public virtual <xsl:value-of select='itemType' /> this[int index]
		{
			get
			{
				ValidateIndex(index); // throws
				return m_array[index]; 
			}
			set
			{
				ValidateIndex(index); // throws
				++m_version; 
				m_array[index] = value; 
			}
		}

		/// &lt;summary&gt;
		///		Adds a &lt;see cref="<xsl:value-of select="itemType"/>"/&gt; to the end of the &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt;.
		/// &lt;/summary&gt;
		/// &lt;param name="item"&gt;The &lt;see cref="<xsl:value-of select="itemType"/>"/&gt; to be added to the end of the &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt;.&lt;/param&gt;
		/// &lt;returns&gt;The index at which the value has been added.&lt;/returns&gt;
		public virtual int Add(<xsl:value-of select='itemType' /> item)
		{
			if (m_count == m_array.Length)
				EnsureCapacity(m_count + 1);

			m_array[m_count] = item;
			m_version++;

			return m_count++;
		}
		
		/// &lt;summary&gt;
		///		Removes all elements from the &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt;.
		/// &lt;/summary&gt;
		public virtual void Clear()
		{
			++m_version;
			m_array = new <xsl:value-of select='itemType' />[DEFAULT_CAPACITY];
			m_count = 0;
		}
		
		/// &lt;summary&gt;
		///		Creates a shallow copy of the &lt;see cref="<xsl:value-of select='collectionName' />"/&gt;.
		/// &lt;/summary&gt;
		public virtual object Clone()
		{
			<xsl:value-of select='collectionName' /> newColl = new <xsl:value-of select='collectionName' />(m_count);
			Array.Copy(m_array, 0, newColl.m_array, 0, m_count);
			newColl.m_count = m_count;
			newColl.m_version = m_version;

			return newColl;
		}

		/// &lt;summary&gt;
		///		Determines whether a given &lt;see cref="<xsl:value-of select="itemType"/>"/&gt; is in the &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt;.
		/// &lt;/summary&gt;
		/// &lt;param name="item"&gt;The &lt;see cref="<xsl:value-of select="itemType"/>"/&gt; to check for.&lt;/param&gt;
		/// &lt;returns&gt;&lt;c&gt;true&lt;/c&gt; if &lt;paramref name="item"/&gt; is found in the &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt;; otherwise, &lt;c&gt;false&lt;/c&gt;.&lt;/returns&gt;
		public virtual bool Contains(<xsl:value-of select='itemType' /> item)
		{
			for (int i=0; i != m_count; ++i)
				if (m_array[i].Equals(item))
					return true;
			return false;
		}

		/// &lt;summary&gt;
		///		Returns the zero-based index of the first occurrence of a &lt;see cref="<xsl:value-of select="itemType"/>"/&gt;
		///		in the &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt;.
		/// &lt;/summary&gt;
		/// &lt;param name="item"&gt;The &lt;see cref="<xsl:value-of select="itemType"/>"/&gt; to locate in the &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt;.&lt;/param&gt;
		/// &lt;returns&gt;
		///		The zero-based index of the first occurrence of &lt;paramref name="item"/&gt; 
		///		in the entire &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt;, if found; otherwise, -1.
		///	&lt;/returns&gt;
		public virtual int IndexOf(<xsl:value-of select='itemType' /> item)
		{
			for (int i=0; i != m_count; ++i)
				if (m_array[i].Equals(item))
					return i;
			return -1;
		}

		/// &lt;summary&gt;
		///		Inserts an element into the &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt; at the specified index.
		/// &lt;/summary&gt;
		/// &lt;param name="index"&gt;The zero-based index at which &lt;paramref name="item"/> should be inserted.&lt;/param&gt;
		/// &lt;param name="item"&gt;The &lt;see cref="<xsl:value-of select="itemType"/>"/&gt; to insert.&lt;/param&gt;
		/// &lt;exception cref="ArgumentOutOfRangeException"&gt;
		///		&lt;para&gt;&lt;paramref name="index"/&gt; is less than zero&lt;/para&gt;
		///		&lt;para&gt;-or-&lt;/para&gt;
		///		&lt;para&gt;&lt;paramref name="index"/&gt; is equal to or greater than &lt;see cref="<xsl:value-of select="collectionName"/>.Count"/&gt;.&lt;/para&gt;
		/// &lt;/exception&gt;
		public virtual void Insert(int index, <xsl:value-of select='itemType' /> item)
		{
			ValidateIndex(index, true); // throws
			
			if (m_count == m_array.Length)
				EnsureCapacity(m_count + 1);

			if (index &lt; m_count)
			{
				Array.Copy(m_array, index, m_array, index + 1, m_count - index);
			}

			m_array[index] = item;
			m_count++;
			m_version++;
		}

		/// &lt;summary&gt;
		///		Removes the first occurrence of a specific &lt;see cref="<xsl:value-of select="itemType"/>"/&gt; from the &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt;.
		/// &lt;/summary&gt;
		/// &lt;param name="item"&gt;The &lt;see cref="<xsl:value-of select="itemType"/>"/&gt; to remove from the &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt;.&lt;/param&gt;
		/// &lt;exception cref="ArgumentException"&gt;
		///		The specified &lt;see cref="<xsl:value-of select="itemType"/>"/&gt; was not found in the &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt;.
		/// &lt;/exception&gt;
		public virtual void Remove(<xsl:value-of select='itemType' /> item)
		{		   
			int i = IndexOf(item);
			if (i &lt; 0)
				throw new System.ArgumentException("Cannot remove the specified item because it was not found in the specified Collection.");
			
			++m_version;
			RemoveAt(i);
		}

		/// &lt;summary&gt;
		///		Removes the element at the specified index of the &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt;.
		/// &lt;/summary&gt;
		/// &lt;param name="index"&gt;The zero-based index of the element to remove.&lt;/param&gt;
		/// &lt;exception cref="ArgumentOutOfRangeException"&gt;
		///		&lt;para&gt;&lt;paramref name="index"/&gt; is less than zero&lt;/para&gt;
		///		&lt;para&gt;-or-&lt;/para&gt;
		///		&lt;para&gt;&lt;paramref name="index"/&gt; is equal to or greater than &lt;see cref="<xsl:value-of select="collectionName"/>.Count"/&gt;.&lt;/para&gt;
		/// &lt;/exception&gt;
		public virtual void RemoveAt(int index)
		{
			ValidateIndex(index); // throws
			
			m_count--;

			if (index &lt; m_count)
			{
				Array.Copy(m_array, index + 1, m_array, index, m_count - index);
			}
			
			// We can't set the deleted entry equal to null, because it might be a value type.
			// Instead, we'll create an empty single-element array of the right type and copy it 
			// over the entry we want to erase.
			<xsl:value-of select='itemType' />[] temp = new <xsl:value-of select='itemType' />[1];
			Array.Copy(temp, 0, m_array, m_count, 1);
			m_version++;
		}

		/// &lt;summary&gt;
		///		Gets a value indicating whether the collection has a fixed size.
		/// &lt;/summary&gt;
		/// &lt;value&gt;true if the collection has a fixed size; otherwise, false. The default is false&lt;/value&gt;
        public virtual bool IsFixedSize
        {
            get { return false; }
        }

		/// &lt;summary&gt;
		///		gets a value indicating whether the <B>IList</B> is read-only.
		/// &lt;/summary&gt;
		/// &lt;value&gt;true if the collection is read-only; otherwise, false. The default is false&lt;/value&gt;
        public virtual bool IsReadOnly
        {
            get { return false; }
        }
		#endregion

		#region Operations (type-safe IEnumerable)
		
		/// &lt;summary&gt;
		///		Returns an enumerator that can iterate through the &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt;.
		/// &lt;/summary&gt;
		/// &lt;returns&gt;An &lt;see cref="Enumerator"/&gt; for the entire &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt;.&lt;/returns&gt;
		public virtual I<xsl:value-of select='collectionName'/>Enumerator GetEnumerator()
		{
			return new Enumerator(this);
		}
		#endregion

		#region Public helpers (just to mimic some nice features of ArrayList)
		
		/// &lt;summary&gt;
		///		Gets or sets the number of elements the &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt; can contain.
		/// &lt;/summary&gt;
		public virtual int Capacity
		{
			get { return m_array.Length; }
			
			set
			{
				if (value &lt; m_count)
					value = m_count;

				if (value != m_array.Length)
				{
					if (value &gt; 0)
					{
						<xsl:value-of select='itemType' />[] temp = new <xsl:value-of select='itemType' />[value];
						Array.Copy(m_array, temp, m_count);
						m_array = temp;
					}
					else
					{
						m_array = new <xsl:value-of select='itemType' />[DEFAULT_CAPACITY];
					}
				}
			}
		}

		/// &lt;summary&gt;
		///		Adds the elements of another &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt; to the current &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt;.
		/// &lt;/summary&gt;
		/// &lt;param name="x"&gt;The &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt; whose elements should be added to the end of the current &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt;.&lt;/param&gt;
		/// &lt;returns&gt;The new &lt;see cref="<xsl:value-of select="collectionName"/>.Count"/&gt; of the &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt;.&lt;/returns&gt;
		public virtual int AddRange(<xsl:value-of select='collectionName' /> x)
		{
			if (m_count + x.Count &gt;= m_array.Length)
				EnsureCapacity(m_count + x.Count);
			
			Array.Copy(x.m_array, 0, m_array, m_count, x.Count);
			m_count += x.Count;
			m_version++;

			return m_count;
		}

		/// &lt;summary&gt;
		///		Adds the elements of a &lt;see cref="<xsl:value-of select="itemType"/>"/&gt; array to the current &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt;.
		/// &lt;/summary&gt;
		/// &lt;param name="x"&gt;The &lt;see cref="<xsl:value-of select="itemType"/>"/&gt; array whose elements should be added to the end of the &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt;.&lt;/param&gt;
		/// &lt;returns&gt;The new &lt;see cref="<xsl:value-of select="collectionName"/>.Count"/&gt; of the &lt;c&gt;<xsl:value-of select='collectionName' />&lt;/c&gt;.&lt;/returns&gt;
		public virtual int AddRange(<xsl:value-of select='itemType' />[] x)
		{
			if (m_count + x.Length &gt;= m_array.Length)
				EnsureCapacity(m_count + x.Length);

			Array.Copy(x, 0, m_array, m_count, x.Length);
			m_count += x.Length;
			m_version++;

			return m_count;
		}
		
		/// &lt;summary&gt;
		///		Sets the capacity to the actual number of elements.
		/// &lt;/summary&gt;
		public virtual void TrimToSize()
		{
			this.Capacity = m_count;
		}

		#endregion

		#region Implementation (helpers)

		/// &lt;exception cref="ArgumentOutOfRangeException"&gt;
		///		&lt;para&gt;&lt;paramref name="index"/&gt; is less than zero&lt;/para&gt;
		///		&lt;para&gt;-or-&lt;/para&gt;
		///		&lt;para&gt;&lt;paramref name="index"/&gt; is equal to or greater than &lt;see cref="<xsl:value-of select="collectionName"/>.Count"/&gt;.&lt;/para&gt;
		/// &lt;/exception&gt;
		private void ValidateIndex(int i)
		{
			ValidateIndex(i, false);
		}

		/// &lt;exception cref="ArgumentOutOfRangeException"&gt;
		///		&lt;para&gt;&lt;paramref name="index"/&gt; is less than zero&lt;/para&gt;
		///		&lt;para&gt;-or-&lt;/para&gt;
		///		&lt;para&gt;&lt;paramref name="index"/&gt; is equal to or greater than &lt;see cref="<xsl:value-of select="collectionName"/>.Count"/&gt;.&lt;/para&gt;
		/// &lt;/exception&gt;
		private void ValidateIndex(int i, bool allowEqualEnd)
		{
			int max = (allowEqualEnd)?(m_count):(m_count-1);
			if (i &lt; 0 || i &gt; max)
				throw new System.ArgumentOutOfRangeException("Index was out of range.  Must be non-negative and less than the size of the collection.", (object)i, "Specified argument was out of the range of valid values.");
		}

		private void EnsureCapacity(int min)
		{
			int newCapacity = ((m_array.Length == 0) ? DEFAULT_CAPACITY : m_array.Length * 2);
			if (newCapacity &lt; min)
				newCapacity = min;

			this.Capacity = newCapacity;
		}

		#endregion
		
		#region Implementation (ICollection)

		void ICollection.CopyTo(Array array, int start)
		{
			Array.Copy(m_array, 0, array, start, m_count);
		}

		#endregion

		#region Implementation (IList)

		object IList.this[int i]
		{
			get { return (object)this[i]; }
			set { this[i] = (<xsl:value-of select='itemType' />)value; }
		}

		int IList.Add(object x)
		{
			return this.Add((<xsl:value-of select='itemType' />)x);
		}

    	bool IList.Contains(object x)
		{
			return this.Contains((<xsl:value-of select='itemType' />)x);
		}

		int IList.IndexOf(object x)
		{
			return this.IndexOf((<xsl:value-of select='itemType' />)x);
		}

		void IList.Insert(int pos, object x)
		{
			this.Insert(pos, (<xsl:value-of select='itemType' />)x);
		}

		void IList.Remove(object x)
		{
			this.Remove((<xsl:value-of select='itemType' />)x);
		}

		void IList.RemoveAt(int pos)
		{
			this.RemoveAt(pos);
		}

		#endregion

		#region Implementation (IEnumerable)

		IEnumerator IEnumerable.GetEnumerator()
		{
			return (IEnumerator)(this.GetEnumerator());
		}

		#endregion

		#region Nested enumerator class
		/// &lt;summary&gt;
		///		Supports simple iteration over a &lt;see cref="<xsl:value-of select="collectionName"/>"/&gt;.
		/// &lt;/summary&gt;
		private class Enumerator : IEnumerator, I<xsl:value-of select='collectionName'/>Enumerator
		{
			#region Implementation (data)
			
			private <xsl:value-of select='collectionName' /> m_collection;
			private int m_index;
			private int m_version;
			
			#endregion
		
			#region Construction
			
			/// &lt;summary&gt;
			///		Initializes a new instance of the &lt;c&gt;Enumerator&lt;/c&gt; class.
			/// &lt;/summary&gt;
			/// &lt;param name="tc"&gt;&lt;/param&gt;
			internal Enumerator(<xsl:value-of select='collectionName' /> tc)
			{
				m_collection = tc;
				m_index = -1;
				m_version = tc.m_version;
			}
			
			#endregion
	
			#region Operations (type-safe IEnumerator)
			
			/// &lt;summary&gt;
			///		Gets the current element in the collection.
			/// &lt;/summary&gt;
			public <xsl:value-of select='itemType' /> Current
			{
				get { return m_collection[m_index]; }
			}

			/// &lt;summary&gt;
			///		Advances the enumerator to the next element in the collection.
			/// &lt;/summary&gt;
			/// &lt;exception cref="InvalidOperationException"&gt;
			///		The collection was modified after the enumerator was created.
			/// &lt;/exception&gt;
			/// &lt;returns&gt;
			///		&lt;c&gt;true&lt;/c&gt; if the enumerator was successfully advanced to the next element; 
			///		&lt;c&gt;false&lt;/c&gt; if the enumerator has passed the end of the collection.
			/// &lt;/returns&gt;
			public bool MoveNext()
			{
				if (m_version != m_collection.m_version)
					throw new System.InvalidOperationException("Collection was modified; enumeration operation may not execute.");

				++m_index;
				return (m_index &lt; m_collection.Count) ? true : false;
			}

			/// &lt;summary&gt;
			///		Sets the enumerator to its initial position, before the first element in the collection.
			/// &lt;/summary&gt;
			public void Reset()
			{
				m_index = -1;
			}
			#endregion
	
			#region Implementation (IEnumerator)
			
			object IEnumerator.Current
			{
				get { return (object)(this.Current); }
			}
			
			#endregion
		}
        #endregion
        
        #region Nested Syncronized Wrapper class
        private class Sync<xsl:value-of select='collectionName'/> : <xsl:value-of select='collectionName'/>
        {
            #region Implementation (data)
            private <xsl:value-of select='collectionName'/> m_collection;
            private object m_root;
            #endregion

            #region Construction
            internal Sync<xsl:value-of select='collectionName'/>(<xsl:value-of select='collectionName'/> list) : base(Tag.Default)
            {
                m_root = list.SyncRoot;
                m_collection = list;
            }
            #endregion
            
            #region Type-safe ICollection
            public override void CopyTo(<xsl:value-of select='itemType' />[] array)
            {
                lock(this.m_root)
                    m_collection.CopyTo(array);
            }

            public override void CopyTo(<xsl:value-of select='itemType' />[] array, int start)
            {
                lock(this.m_root)
                    m_collection.CopyTo(array,start);
            }
            public override int Count
            {
                get
                { 
                    lock(this.m_root)
                        return m_collection.Count;
                }
            }

            public override bool IsSynchronized
            {
                get { return true; }
            }

            public override object SyncRoot
            {
                get { return this.m_root; }
            }
            #endregion
            
            #region Type-safe IList
            public override <xsl:value-of select='itemType' /> this[int i]
            {
                get
                {
                    lock(this.m_root)
                        return m_collection[i];
                }
                set
                {
                    lock(this.m_root)
                        m_collection[i] = value; 
                }
            }

            public override int Add(<xsl:value-of select='itemType' /> x)
            {
                lock(this.m_root)
                    return m_collection.Add(x);
            }
            
            public override void Clear()
            {
                lock(this.m_root)
                    m_collection.Clear();
            }

            public override bool Contains(<xsl:value-of select='itemType' /> x)
            {
                lock(this.m_root)
                    return m_collection.Contains(x);
            }

            public override int IndexOf(<xsl:value-of select='itemType' /> x)
            {
                lock(this.m_root)
                    return m_collection.IndexOf(x);
            }

            public override void Insert(int pos, <xsl:value-of select='itemType' /> x)
            {
                lock(this.m_root)
                    m_collection.Insert(pos,x);
            }

            public override void Remove(<xsl:value-of select='itemType' /> x)
            {           
                lock(this.m_root)
                    m_collection.Remove(x);
            }

            public override void RemoveAt(int pos)
            {
                lock(this.m_root)
                    m_collection.RemoveAt(pos);
            }
            
            public override bool IsFixedSize
            {
                get {return m_collection.IsFixedSize;}
            }

            public override bool IsReadOnly
            {
                get {return m_collection.IsReadOnly;}
            }
            #endregion

            #region Type-safe IEnumerable
            public override I<xsl:value-of select='collectionName'/>Enumerator GetEnumerator()
            {
                lock(m_root)
                    return m_collection.GetEnumerator();
            }
            #endregion

            #region Public Helpers
            // (just to mimic some nice features of ArrayList)
            public override int Capacity
            {
                get
                {
                    lock(this.m_root)
                        return m_collection.Capacity;
                }
                
                set
                {
                    lock(this.m_root)
                        m_collection.Capacity = value;
                }
            }

            public override int AddRange(<xsl:value-of select='collectionName' /> x)
            {
                lock(this.m_root)
                    return m_collection.AddRange(x);
            }

            public override int AddRange(<xsl:value-of select='itemType' />[] x)
            {
                lock(this.m_root)
                    return m_collection.AddRange(x);
            }
            #endregion
        }
        #endregion

        #region Nested Read Only Wrapper class
        private class ReadOnly<xsl:value-of select='collectionName'/> : <xsl:value-of select='collectionName'/>
        {
            #region Implementation (data)
            private <xsl:value-of select='collectionName'/> m_collection;
            #endregion

            #region Construction
            internal ReadOnly<xsl:value-of select='collectionName'/>(<xsl:value-of select='collectionName'/> list) : base(Tag.Default)
            {
                m_collection = list;
            }
            #endregion
            
            #region Type-safe ICollection
            public override void CopyTo(<xsl:value-of select='itemType' />[] array)
            {
                m_collection.CopyTo(array);
            }

            public override void CopyTo(<xsl:value-of select='itemType' />[] array, int start)
            {
                m_collection.CopyTo(array,start);
            }
            public override int Count
            {
                get {return m_collection.Count;}
            }

            public override bool IsSynchronized
            {
                get { return m_collection.IsSynchronized; }
            }

            public override object SyncRoot
            {
                get { return this.m_collection.SyncRoot; }
            }
            #endregion
            
            #region Type-safe IList
            public override <xsl:value-of select='itemType' /> this[int i]
            {
                get { return m_collection[i]; }
                set { throw new NotSupportedException("This is a Read Only Collection and can not be modified"); }
            }

            public override int Add(<xsl:value-of select='itemType' /> x)
            {
                throw new NotSupportedException("This is a Read Only Collection and can not be modified");
            }
            
            public override void Clear()
            {
                throw new NotSupportedException("This is a Read Only Collection and can not be modified");
            }

            public override bool Contains(<xsl:value-of select='itemType' /> x)
            {
                return m_collection.Contains(x);
            }

            public override int IndexOf(<xsl:value-of select='itemType' /> x)
            {
                return m_collection.IndexOf(x);
            }

            public override void Insert(int pos, <xsl:value-of select='itemType' /> x)
            {
                throw new NotSupportedException("This is a Read Only Collection and can not be modified");
            }

            public override void Remove(<xsl:value-of select='itemType' /> x)
            {           
                throw new NotSupportedException("This is a Read Only Collection and can not be modified");
            }

            public override void RemoveAt(int pos)
            {
                throw new NotSupportedException("This is a Read Only Collection and can not be modified");
            }
            
            public override bool IsFixedSize
            {
                get {return true;}
            }

            public override bool IsReadOnly
            {
                get {return true;}
            }
            #endregion

            #region Type-safe IEnumerable
            public override I<xsl:value-of select='collectionName'/>Enumerator GetEnumerator()
            {
                return m_collection.GetEnumerator();
            }
            #endregion

            #region Public Helpers
            // (just to mimic some nice features of ArrayList)
            public override int Capacity
            {
                get { return m_collection.Capacity; }
                
                set { throw new NotSupportedException("This is a Read Only Collection and can not be modified"); }
            }

            public override int AddRange(<xsl:value-of select='collectionName' /> x)
            {
                throw new NotSupportedException("This is a Read Only Collection and can not be modified");
            }

            public override int AddRange(<xsl:value-of select='itemType' />[] x)
            {
                throw new NotSupportedException("This is a Read Only Collection and can not be modified");
            }
            #endregion
        }
        #endregion
	}
<xsl:if test="collectionNamespace">
	<xsl:if test="string-length(collectionNamespace) > 0">
}
	</xsl:if>
</xsl:if>

  </xsl:template>

</xsl:transform>
