<!-- CSVector.xslt -->
<!-- This code leveraged from Shawn Van Ness [shawnv@arithex.com] -->
<!-- http://www.develop.com/software/GenXdownloaddetail.asp?vWizId=281 -->
<!-- Updated by Matthias Hess [matthias.hess@bluewin.ch] to fix XSLT "garbage" problem -->
<!-- This code ported from C# by Tor-Erik Hagen [tor-erik@web-amp.com] -->
<xsl:transform version='1.0' xmlns:xsl='http://www.w3.org/1999/XSL/Transform'>
  <xsl:output method='text'/>
  
  <xsl:template match="/">
	<xsl:apply-templates select='/typeSafeCollections/typeSafeCollection[templateKind="Vector"]'/>
  </xsl:template>

  <xsl:template match='/typeSafeCollections/typeSafeCollection[templateKind="Vector"]'>

<xsl:if test="collectionNamespace">
	<xsl:if test="string-length(collectionNamespace) > 0">
Namespace <xsl:value-of select='collectionNamespace' />
	</xsl:if>
</xsl:if>

    '/// &lt;summary&gt;
    '///		A strongly-typed collection of &lt;see cref="<xsl:value-of select="itemType"/>"/> objects.
    '/// &lt;/summary&gt;
    &lt;Serializable()&gt; _
    Public Class <xsl:value-of select="collectionName"/>
        Implements ICollection
        Implements IList
        Implements IEnumerable
        Implements ICloneable

#Region "Interfaces"
        '/// &lt;summary&gt;
        '///		Supports type-safe iteration over a &lt;see cref="<xsl:value-of select="collectionName"/>"/>.
        '/// &lt;/summary&gt;
        Public Interface I<xsl:value-of select="collectionName"/>Enumerator

            '/// &lt;summary&gt;
            '///		Gets the current element in the collection.
            '/// &lt;/summary&gt;
            ReadOnly Property Current() As <xsl:value-of select="itemType"/>

            '/// &lt;summary&gt;
            '///		Advances the enumerator to the next element in the collection.
            '/// &lt;/summary&gt;
            '/// &lt;exception cref="InvalidOperationException"&gt;
            '///		The collection was modified after the enumerator was created.
            '/// &lt;/exception&gt;
            '/// &lt;returns&gt;
            '///		&lt;c&gt;true&lt;/c&gt; if the enumerator was successfully advanced to the next element 
            '///		&lt;c&gt;false&lt;/c&gt; if the enumerator has passed the end of the collection.
            '/// &lt;/returns&gt;
            Function MoveNext() As Boolean

            '/// &lt;summary&gt;
            '///		Sets the enumerator to its initial position, before the first element in the collection.
            '/// &lt;/summary&gt;
            Sub Reset()
        End Interface
#End Region

        Private Const DEFAULT_CAPACITY As Integer = 15

#Region "Implementation (data)"
        Private m_array() As <xsl:value-of select="itemType"/>
        Private m_count As Integer = 0
        &lt;NonSerialized()&gt; _
        Private m_version As Integer = 0
#End Region


#Region "Static Wrappers"
        '/// &lt;summary&gt;
        '///		Creates a synchronized (thread-safe) wrapper for a 
        '///     &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt; instance.
        '/// &lt;/summary&gt;
        '/// &lt;returns&gt;
        '///     An &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt; wrapper that is synchronized (thread-safe).
        '/// &lt;/returns&gt;
        Public Shared Function Synchronized(ByVal list As <xsl:value-of select="collectionName"/>) As <xsl:value-of select="collectionName"/>
            If (list Is Nothing) Then
                Throw New ArgumentNullException("list")
            End If
            Return New Sync<xsl:value-of select="collectionName"/>(list)
        End Function

        '/// &lt;summary&gt;
        '///		Creates a read-only wrapper for a 
        '///     &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt; instance.
        '/// &lt;/summary&gt;
        '/// &lt;returns&gt;
        '///     An &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt; wrapper that is read-only.
        '/// &lt;/returns&gt;
        Public Shared Function GetReadOnly(ByVal list As <xsl:value-of select="collectionName"/>) As <xsl:value-of select="collectionName"/>
            If (list Is Nothing) Then
                Throw New ArgumentNullException("list")
            End If
            Return New ReadOnly<xsl:value-of select="collectionName"/>(list)
        End Function
#End Region

#Region "Construction"
        '/// &lt;summary&gt;
        '///		Initializes a new instance of the &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt; class
        '///		that is empty and has the default initial capacity.
        '/// &lt;/summary&gt;
        Public Sub New()
            m_array = New <xsl:value-of select="itemType"/>(DEFAULT_CAPACITY) {}
        End Sub

        '/// &lt;summary&gt;
        '///		Initializes a new instance of the &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt; class
        '///		that has the specified initial capacity.
        '/// &lt;/summary&gt;
        '/// &lt;param name="capacity"&gt;
        '///		The number of elements that the new &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt; is initially capable of storing.
        '///	&lt;/param&gt;
        Public Sub New(ByVal capacity As Integer)
            m_array = New <xsl:value-of select="itemType"/>(capacity) {}
        End Sub

        '/// &lt;summary&gt;
        '///		Initializes a new instance of the &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt; class
        '///		that contains elements copied from the specified &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt;.
        '/// &lt;/summary&gt;
        '/// &lt;param name="c"&gt;The &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt; whose elements are copied to the new collection.&lt;/param&gt;
        Public Sub New(ByVal c As <xsl:value-of select="collectionName"/>)
            m_array = New <xsl:value-of select="itemType"/>(c.Count) {}
            AddRange(c)
        End Sub

        '/// &lt;summary&gt;
        '///		Initializes a new instance of the &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt; class
        '///		that contains elements copied from the specified &lt;see cref="<xsl:value-of select="itemType"/>"/> array.
        '/// &lt;/summary&gt;
        '/// &lt;param name="a"&gt;The &lt;see cref="<xsl:value-of select="itemType"/>"/> array whose elements are copied to the new list.&lt;/param&gt;
        Public Sub New(ByVal a() As <xsl:value-of select="itemType"/>)
            m_array = New <xsl:value-of select="itemType"/>(a.Length) {}
            AddRange(a)
        End Sub
#End Region

#Region "Operations (type-safe ICollection)"
        '/// &lt;summary&gt;
        '///		Gets the number of elements actually contained in the &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt;.
        '/// &lt;/summary&gt;
        Public Overridable ReadOnly Property Count() As Integer Implements ICollection.Count
            Get
                Return m_count
            End Get
        End Property

        '/// &lt;summary&gt;
        '///		Copies the entire &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt; to a one-dimensional
        '///		&lt;see cref="<xsl:value-of select="itemType"/>"/&gt; array.
        '/// &lt;/summary&gt;
        '/// &lt;param name="array"&gt;The one-dimensional &lt;see cref="<xsl:value-of select="itemType"/>"/> array to copy to.&lt;/param&gt;
        Public Overridable Sub CopyTo(ByVal array() As <xsl:value-of select="itemType"/>)
            Me.CopyTo(array, 0)
        End Sub

        '/// &lt;summary&gt;
        '///		Copies the entire &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt; to a one-dimensional
        '///		&lt;see cref="<xsl:value-of select="itemType"/>"/> array, starting at the specified index of the target array.
        '/// &lt;/summary&gt;
        '/// &lt;param name="array"&gt;The one-dimensional &lt;see cref="<xsl:value-of select="itemType"/>"/> array to copy to.&lt;/param&gt;
        '/// &lt;param name="start"&gt;The zero-based index in &lt;paramref name="array"/> at which copying begins.&lt;/param&gt;
        Public Overridable Sub CopyTo(ByVal array() As <xsl:value-of select="itemType"/>, ByVal start As Integer)
            If (m_count &gt; (array.GetUpperBound(0) + 1 - start)) Then
                Throw New System.ArgumentException("Destination array was not long enough.")
            End If
            array.Copy(m_array, 0, array, start, m_count)
        End Sub

        '/// &lt;summary&gt;
        '///		Gets a value indicating whether access to the collection is synchronized (thread-safe).
        '/// &lt;/summary&gt;
        '/// &lt;returns&gt;true if access to the ICollection is synchronized (thread-safe) otherwise, false.&lt;/returns&gt;
        Public Overridable ReadOnly Property IsSynchronized() As Boolean Implements ICollection.IsSynchronized
            Get
                Return m_array.IsSynchronized
            End Get
        End Property

        '/// &lt;summary&gt;
        '///		Gets an object that can be used to synchronize access to the collection.
        '/// &lt;/summary&gt;
        Public Overridable ReadOnly Property SyncRoot() As Object Implements ICollection.SyncRoot
            Get
                Return m_array.SyncRoot
            End Get
        End Property
#End Region


#Region "Operations (type-safe IList)"
        '/// &lt;summary&gt;
        '///		Gets or sets the &lt;see cref="<xsl:value-of select="itemType"/>"/> at the specified index.
        '/// &lt;/summary&gt;
        '/// &lt;param name="index"&gt;The zero-based index of the element to get or set.&lt;/param&gt;
        '/// &lt;exception cref="ArgumentOutOfRangeException"&gt;
        '///		&lt;para&gt;&lt;paramref name="index"/> is less than zero&lt;/para&gt;
        '///		&lt;para&gt;-or-&lt;/para&gt;
        '///		&lt;para&gt;&lt;paramref name="index"/> is equal to or greater than &lt;see cref="<xsl:value-of select="collectionName"/>.Count"/>.&lt;/para&gt;
        '/// &lt;/exception&gt;
        Default Public Overridable Property Item(ByVal index As Integer) As <xsl:value-of select="itemType"/>
            Get
                ValidateIndex(index) '// throws
                Return m_array(index)
            End Get
            Set(ByVal Value As <xsl:value-of select="itemType"/>)
                ValidateIndex(index) '// throws
                m_version += 1
                m_array(index) = Value
            End Set
        End Property

        '/// &lt;summary&gt;
        '///		Adds a &lt;see cref="<xsl:value-of select="itemType"/>"/> to the end of the &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt;.
        '/// &lt;/summary&gt;
        '/// &lt;param name="item"&gt;The &lt;see cref="<xsl:value-of select="itemType"/>"/> to be added to the end of the &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt;.&lt;/param&gt;
        '/// &lt;returns&gt;The index at which the value has been added.&lt;/returns&gt;
        Public Overridable Function Add(ByVal item As <xsl:value-of select="itemType"/>) As Integer
            If (m_count = m_array.Length) Then
                EnsureCapacity(m_count + 1)
            End If

            m_array(m_count) = item
            m_version += 1
            m_count += 1
            Return (m_count - 1)
        End Function

        '/// &lt;summary&gt;
        '///		Removes all elements from the &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt;.
        '/// &lt;/summary&gt;
        Public Overridable Sub Clear() Implements IList.Clear
            m_version += 1
            m_array = New <xsl:value-of select="itemType"/>(DEFAULT_CAPACITY) {}
            m_count = 0
        End Sub

        '/// &lt;summary&gt;
        '///		Creates a shallow copy of the &lt;see cref="<xsl:value-of select="collectionName"/>"/>.
        '/// &lt;/summary&gt;
        Public Overridable Function Clone() As Object Implements ICloneable.Clone
            Dim newColl As <xsl:value-of select="collectionName"/> = New <xsl:value-of select="collectionName"/>(m_count)
            Array.Copy(m_array, 0, newColl.m_array, 0, m_count)
            newColl.m_count = m_count
            newColl.m_version = m_version

            Return newColl
        End Function

        '/// &lt;summary&gt;
        '///		Determines whether a given &lt;see cref="<xsl:value-of select="itemType"/>"/> is in the &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt;.
        '/// &lt;/summary&gt;
        '/// &lt;param name="item"&gt;The &lt;see cref="<xsl:value-of select="itemType"/>"/> to check for.&lt;/param&gt;
        '/// &lt;returns&gt;&lt;c&gt;true&lt;/c&gt; if &lt;paramref name="item"/> is found in the &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt; otherwise, &lt;c&gt;false&lt;/c&gt;.&lt;/returns&gt;
        Public Overridable Function Contains(ByVal item As <xsl:value-of select="itemType"/>) As Boolean
            Dim i As Integer
            For i = 0 To (m_count - 1) Step 1
                If (m_array(i).Equals(item)) Then
                    Return True
                End If
            Next
            Return False
        End Function

        '/// &lt;summary&gt;
        '///		Returns the zero-based index of the first occurrence of a &lt;see cref="<xsl:value-of select="itemType"/>"/>
        '///		in the &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt;.
        '/// &lt;/summary&gt;
        '/// &lt;param name="item"&gt;The &lt;see cref="<xsl:value-of select="itemType"/>"/> to locate in the &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt;.&lt;/param&gt;
        '/// &lt;returns&gt;
        '///		The zero-based index of the first occurrence of &lt;paramref name="item"/> 
        '///		in the entire &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt;, if found otherwise, -1.
        '///	&lt;/returns&gt;
        Public Overridable Function IndexOf(ByVal item As <xsl:value-of select="itemType"/>) As Integer
            Dim i As Integer
            For i = 0 To (m_count - 1) Step 1
                If (m_array(i).Equals(item)) Then
                    Return i
                End If
            Next
            Return -1
        End Function

        '/// &lt;summary&gt;
        '///		Inserts an element into the &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt; at the specified index.
        '/// &lt;/summary&gt;
        '/// &lt;param name="index"&gt;The zero-based index at which &lt;paramref name="item"/> should be inserted.&lt;/param&gt;
        '/// &lt;param name="item"&gt;The &lt;see cref="<xsl:value-of select="itemType"/>"/> to insert.&lt;/param&gt;
        '/// &lt;exception cref="ArgumentOutOfRangeException"&gt;
        '///		&lt;para&gt;&lt;paramref name="index"/> is less than zero&lt;/para&gt;
        '///		&lt;para&gt;-or-&lt;/para&gt;
        '///		&lt;para&gt;&lt;paramref name="index"/> is equal to or greater than &lt;see cref="<xsl:value-of select="collectionName"/>.Count"/>.&lt;/para&gt;
        '/// &lt;/exception&gt;
        Public Overridable Sub Insert(ByVal index As Integer, ByVal item As <xsl:value-of select="itemType"/>)
            ValidateIndex(index, True) '// throws

            If (m_count = m_array.Length) Then EnsureCapacity(m_count + 1)

            If (index &lt; m_count) Then
                Array.Copy(m_array, index, m_array, index + 1, m_count - index)
            End If

            m_array(index) = item
            m_count += 1
            m_version += 1
        End Sub

        '/// &lt;summary&gt;
        '///		Removes the first occurrence of a specific &lt;see cref="<xsl:value-of select="itemType"/>"/> from the &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt;.
        '/// &lt;/summary&gt;
        '/// &lt;param name="item"&gt;The &lt;see cref="<xsl:value-of select="itemType"/>"/> to remove from the &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt;.&lt;/param&gt;
        '/// &lt;exception cref="ArgumentException"&gt;
        '///		The specified &lt;see cref="<xsl:value-of select="itemType"/>"/> was not found in the &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt;.
        '/// &lt;/exception&gt;
        Public Overridable Sub Remove(ByVal item As <xsl:value-of select="itemType"/>)
            Dim i As Integer = IndexOf(item)
            If (i &lt; 0) Then
                Throw New System.ArgumentException("Cannot remove the specified item because it was not found in the specified Collection.")
            End If

            m_version += 1
            RemoveAt(i)
        End Sub

        '/// &lt;summary&gt;
        '///		Removes the element at the specified index of the &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt;.
        '/// &lt;/summary&gt;
        '/// &lt;param name="index"&gt;The zero-based index of the element to remove.&lt;/param&gt;
        '/// &lt;exception cref="ArgumentOutOfRangeException"&gt;
        '///		&lt;para&gt;&lt;paramref name="index"/> is less than zero&lt;/para&gt;
        '///		&lt;para&gt;-or-&lt;/para&gt;
        '///		&lt;para&gt;&lt;paramref name="index"/> is equal to or greater than &lt;see cref="<xsl:value-of select="collectionName"/>.Count"/>.&lt;/para&gt;
        '/// &lt;/exception&gt;
        Public Overridable Sub RemoveAt(ByVal index As Integer) Implements IList.RemoveAt

            ValidateIndex(index) '// throws

            m_count -= 1

            If (index &lt; m_count) Then
                Array.Copy(m_array, index + 1, m_array, index, m_count - index)
            End If

            '// We can't set the deleted entry equal to null, because it might be a value type.
            '// Instead, we'll create an empty single-element array of the right type and copy it 
            '// over the entry we want to erase.
            Dim temp(0) As <xsl:value-of select="itemType"/>
            temp(0) = New <xsl:value-of select="itemType"/>()
            Array.Copy(temp, 0, m_array, m_count, 1)
            m_version += 1
        End Sub

        '/// &lt;summary&gt;
        '///		Gets a value indicating whether the collection has a fixed size.
        '/// &lt;/summary&gt;
        '/// &lt;value&gt;true if the collection has a fixed size otherwise, false. The default is false&lt;/value&gt;
        Public Overridable ReadOnly Property IsFixedSize() As Boolean Implements IList.IsFixedSize
            Get
                Return False
            End Get
        End Property

        '/// &lt;summary&gt;
        '///		gets a value indicating whether the IList is read-only.
        '/// &lt;/summary&gt;
        '/// &lt;value&gt;true if the collection is read-only otherwise, false. The default is false&lt;/value&gt;
        Public Overridable ReadOnly Property IsReadOnly() As Boolean Implements IList.IsReadOnly
            Get
                Return False
            End Get
        End Property
#End Region


#Region "Operations (type-safe IEnumerable)"

        '/// &lt;summary&gt;
        '///		Returns an enumerator that can iterate through the &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt;.
        '/// &lt;/summary&gt;
        '/// &lt;returns&gt;An &lt;see cref="Enumerator"/> for the entire &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt;.&lt;/returns&gt;
        Public Overridable Function GetEnumerator() As I<xsl:value-of select="collectionName"/>Enumerator
            Return New Enumerator(Me)
        End Function
#End Region


#Region "Public helpers (just to mimic some nice features of ArrayList)"

        '/// &lt;summary&gt;
        '///		Gets or sets the number of elements the &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt; can contain.
        '/// &lt;/summary&gt;
        Public Overridable Property Capacity() As Integer
            Get
                Return m_array.Length
            End Get

            Set(ByVal Value As Integer)
                If (Value &lt; m_count) Then Value = m_count

                If Not (Value = m_array.Length) Then
                    If (Value &gt; 0) Then
						' C# implementation
                        'Dim temp() As <xsl:value-of select="itemType"/> = New <xsl:value-of select="itemType"/>(Value-1) {}
                        'Array.Copy(m_array, temp, m_count)
                        'm_array = temp
                        ' VB implementation
                        redim preserve m_array(Value)
                    Else
                        m_array = New <xsl:value-of select="itemType"/>(DEFAULT_CAPACITY) {}
                    End If
                End If
            End Set
        End Property

        '/// &lt;summary&gt;
        '///		Adds the elements of another &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt; to the current &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt;.
        '/// &lt;/summary&gt;
        '/// &lt;param name="x"&gt;The &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt; whose elements should be added to the end of the current &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt;.&lt;/param&gt;
        '/// &lt;returns&gt;The new &lt;see cref="<xsl:value-of select="collectionName"/>.Count"/> of the &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt;.&lt;/returns&gt;
        Public Overridable Function AddRange(ByVal x As <xsl:value-of select="collectionName"/>) As Integer

            If (m_count + x.Count &gt;= m_array.Length) Then EnsureCapacity(m_count + x.Count)

            Array.Copy(x.m_array, 0, m_array, m_count, x.Count)
            m_count += x.Count
            m_version += 1

            Return m_count
        End Function

        '/// &lt;summary&gt;
        '///		Adds the elements of a &lt;see cref="<xsl:value-of select="itemType"/>"/> array to the current &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt;.
        '/// &lt;/summary&gt;
        '/// &lt;param name="x"&gt;The &lt;see cref="<xsl:value-of select="itemType"/>"/> array whose elements should be added to the end of the &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt;.&lt;/param&gt;
        '/// &lt;returns&gt;The new &lt;see cref="<xsl:value-of select="collectionName"/>.Count"/> of the &lt;c&gt;<xsl:value-of select="collectionName"/>&lt;/c&gt;.&lt;/returns&gt;
        Public Overridable Function AddRange(ByVal x() As <xsl:value-of select="itemType"/>) As Integer

            If (m_count + x.Length &gt;= m_array.Length) Then EnsureCapacity(m_count + x.Length)

            Array.Copy(x, 0, m_array, m_count, x.Length)
            m_count += x.Length
            m_version += 1

            Return m_count
        End Function

        '/// &lt;summary&gt;
        '///		Sets the capacity to the actual number of elements.
        '/// &lt;/summary&gt;
        Public Overridable Sub TrimToSize()
            Me.Capacity = m_count
        End Sub

#End Region

#Region "Implementation (helpers)"

        '/// &lt;exception cref="ArgumentOutOfRangeException"&gt;
        '///		&lt;para&gt;&lt;paramref name="index"/> is less than zero&lt;/para&gt;
        '///		&lt;para&gt;-or-&lt;/para&gt;
        '///		&lt;para&gt;&lt;paramref name="index"/> is equal to or greater than &lt;see cref="<xsl:value-of select="collectionName"/>.Count"/>.&lt;/para&gt;
        '/// &lt;/exception&gt;
        Private Sub ValidateIndex(ByVal i As Integer)
            ValidateIndex(i, False)
        End Sub

        '/// &lt;exception cref="ArgumentOutOfRangeException"&gt;
        '///		&lt;para&gt;&lt;paramref name="index"/> is less than zero&lt;/para&gt;
        '///		&lt;para&gt;-or-&lt;/para&gt;
        '///		&lt;para&gt;&lt;paramref name="index"/> is equal to or greater than &lt;see cref="<xsl:value-of select="collectionName"/>.Count"/>.&lt;/para&gt;
        '/// &lt;/exception&gt;
        Private Sub ValidateIndex(ByVal i As Integer, ByVal allowEqualEnd As Boolean)
            Dim max As Integer

            If allowEqualEnd Then
                max = m_count
            Else
                max = m_count - 1
            End If

            If (i &lt; 0 Or i &gt; max) Then
                Throw New System.ArgumentOutOfRangeException("Index was out of range.  Must be non-negative and less than the size of the collection.", CType(1, Object), "Specified argument was out of the range of valid values.")
            End If
        End Sub

        Private Sub EnsureCapacity(ByVal min As Integer)

            Dim newCapacity As Integer
            If m_array.Length = 0 Then
                newCapacity = DEFAULT_CAPACITY
            Else
                newCapacity = m_array.Length * 2
            End If

            If (newCapacity &lt; min) Then newCapacity = min

            Me.Capacity = newCapacity
        End Sub

#End Region

#Region "Implementation (ICollection)"

        Sub CopyTo(ByVal array As Array, ByVal start As Integer) Implements ICollection.CopyTo
            Array.Copy(m_array, 0, array, start, m_count)
        End Sub

#End Region

#Region "Implementation (IList)"
        ' Default removed
        Property IList_Item(ByVal i As Integer) As Object Implements IList.Item
            Get
                Return CType(Me(i), Object)
            End Get

            Set(ByVal Value As Object)
                Me(i) = CType(Value, <xsl:value-of select="itemType"/>)
            End Set
        End Property

        Function Add(ByVal x As Object) As Integer Implements IList.Add
            Return Me.Add(CType(x, <xsl:value-of select="itemType"/>))
        End Function

        Function Contains(ByVal x As Object) As Boolean Implements IList.Contains
            Return Me.Contains(CType(x, <xsl:value-of select="itemType"/>))
        End Function

        Function IndexOf(ByVal x As Object) As Integer Implements IList.IndexOf
            Return Me.IndexOf(CType(x, <xsl:value-of select="itemType"/>))
        End Function

        Sub Insert(ByVal pos As Integer, ByVal x As Object) Implements IList.Insert
            Me.Insert(pos, CType(x, <xsl:value-of select="itemType"/>))
        End Sub

        Sub Remove(ByVal x As Object) Implements IList.Remove
            Me.Remove(CType(x, <xsl:value-of select="itemType"/>))
        End Sub

        'Sub RemoveAt(ByVal pos As Integer) Implements IList.RemoveAt
        '    Me.RemoveAt(pos)
        'End Sub

#End Region

#Region "Implementation (IEnumerable)"

        Function IEnumerable_GetEnumerator() As IEnumerator Implements IEnumerable.GetEnumerator
            Return CType(Me.GetEnumerator(), IEnumerator)
        End Function

#End Region

#Region "Nested enumerator class"
        '/// &lt;summary&gt;
        '///		Supports simple iteration over a &lt;see cref="<xsl:value-of select="collectionName"/>"/>.
        '/// &lt;/summary&gt;
        Private Class Enumerator
            Implements IEnumerator, I<xsl:value-of select="collectionName"/>Enumerator

#Region "Implementation (data)"

            Private m_collection As <xsl:value-of select="collectionName"/>
            Private m_index As Integer
            Private m_version As Integer

#End Region

#Region "Construction"

            '/// &lt;summary&gt;
            '///		Initializes a new instance of the &lt;c&gt;Enumerator&lt;/c&gt; class.
            '/// &lt;/summary&gt;
            '/// &lt;param name="tc"&gt;&lt;/param&gt;
            Friend Sub New(ByVal tc As <xsl:value-of select="collectionName"/>)
                m_collection = tc
                m_index = -1
                m_version = tc.m_version
            End Sub

#End Region

#Region "Operations (type-safe IEnumerator)"

            '/// &lt;summary&gt;
            '///		Gets the current element in the collection.
            '/// &lt;/summary&gt;
            Public ReadOnly Property Current() As <xsl:value-of select="itemType"/> Implements I<xsl:value-of select="collectionName"/>Enumerator.Current
                Get
                    Return m_collection(m_index)
                End Get
            End Property

            '/// &lt;summary&gt;
            '///		Advances the enumerator to the next element in the collection.
            '/// &lt;/summary&gt;
            '/// &lt;exception cref="InvalidOperationException"&gt;
            '///		The collection was modified after the enumerator was created.
            '/// &lt;/exception&gt;
            '/// &lt;returns&gt;
            '///		&lt;c&gt;true&lt;/c&gt; if the enumerator was successfully advanced to the next element 
            '///		&lt;c&gt;false&lt;/c&gt; if the enumerator has passed the end of the collection.
            '/// &lt;/returns&gt;
            Public Function MoveNext() As Boolean Implements IEnumerator.MoveNext, I<xsl:value-of select="collectionName"/>Enumerator.MoveNext
                If Not (m_version = m_collection.m_version) Then
                    Throw New System.InvalidOperationException("Collection was modified enumeration operation may not execute.")
                End If

                m_index += 1

                If m_index &lt; m_collection.Count Then
                    Return True
                Else
                    Return False
                End If
            End Function

            '/// &lt;summary&gt;
            '///		Sets the enumerator to its initial position, before the first element in the collection.
            '/// &lt;/summary&gt;
            Public Sub Reset() Implements IEnumerator.Reset, I<xsl:value-of select="collectionName"/>Enumerator.Reset
                m_index = -1
            End Sub
#End Region

#Region "Implementation (IEnumerator)"

            ReadOnly Property IEnumerator_Current() As Object Implements IEnumerator.Current
                Get
                    Return CType(Me.Current, Object)
                End Get
            End Property

#End Region
        End Class
#End Region

#Region "Nested Syncronized Wrapper class"
        Private Class Sync<xsl:value-of select="collectionName"/>
            Inherits <xsl:value-of select="collectionName"/>

#Region "Implementation (data)"
            Private m_collection As <xsl:value-of select="collectionName"/>
            Private m_root As Object
#End Region

#Region "Construction"
            Friend Sub New(ByVal list As <xsl:value-of select="collectionName"/>)
                m_root = list.SyncRoot
                m_collection = list
            End Sub
#End Region

#Region "Type-safe ICollection"
            
            Public Overloads Overrides Sub CopyTo(ByVal array() As <xsl:value-of select="itemType"/>)
                SyncLock Me.m_root
                    m_collection.CopyTo(array)
                End SyncLock
            End Sub
            
            Public Overloads Overrides Sub CopyTo(ByVal array() As <xsl:value-of select="itemType"/>, ByVal start As Integer)
                SyncLock Me.m_root
                    m_collection.CopyTo(array, start)
                End SyncLock
            End Sub

            Public Overrides ReadOnly Property Count() As Integer
                Get
                    SyncLock Me.m_root
                        Return m_collection.Count
                    End SyncLock
                End Get
            End Property

            Public Overrides ReadOnly Property IsSynchronized() As Boolean
                Get
                    Return True
                End Get
            End Property

            Public Overrides ReadOnly Property SyncRoot() As Object
                Get
                    Return Me.m_root
                End Get
            End Property
#End Region

#Region "Type-safe IList"
            Default Public Overrides Property Item(ByVal i As Integer) As <xsl:value-of select="itemType"/>
                Get
                    SyncLock Me.m_root
                        Return m_collection(i)
                    End SyncLock
                End Get
                Set(ByVal Value As <xsl:value-of select="itemType"/>)
                    SyncLock Me.m_root
                        m_collection(i) = Value
                    End SyncLock
                End Set
            End Property
            
            Public Overloads Overrides Function Add(ByVal x As <xsl:value-of select="itemType"/>) As Integer
                SyncLock Me.m_root
                    Return m_collection.Add(x)
                End SyncLock
            End Function

            Public Overrides Sub Clear()
                SyncLock Me.m_root
                    m_collection.Clear()
                End SyncLock
            End Sub
           
            Public Overloads Overrides Function Contains(ByVal x As <xsl:value-of select="itemType"/>) As Boolean
                SyncLock Me.m_root
                    Return m_collection.Contains(x)
                End SyncLock
            End Function
            
            Public Overloads Overrides Function IndexOf(ByVal x As <xsl:value-of select="itemType"/>) As Integer
                SyncLock Me.m_root
                    Return m_collection.IndexOf(x)
                End SyncLock
            End Function
           
            Public Overloads Overrides Sub Insert(ByVal pos As Integer, ByVal x As <xsl:value-of select="itemType"/>)
                SyncLock Me.m_root
                    m_collection.Insert(pos, x)
                End SyncLock
            End Sub
            
            Public Overloads Overrides Sub Remove(ByVal x As <xsl:value-of select="itemType"/>)
                SyncLock Me.m_root
                    m_collection.Remove(x)
                End SyncLock
            End Sub

            Public Overrides Sub RemoveAt(ByVal pos As Integer)
                SyncLock Me.m_root
                    m_collection.RemoveAt(pos)
                End SyncLock
            End Sub

            Public Overrides ReadOnly Property IsFixedSize() As Boolean
                Get
                    Return m_collection.IsFixedSize
                End Get
            End Property

            Public Overrides ReadOnly Property IsReadOnly() As Boolean
                Get
                    Return m_collection.IsReadOnly
                End Get
            End Property
#End Region

#Region "Type-safe IEnumerable"
            Public Overrides Function GetEnumerator() As I<xsl:value-of select="collectionName"/>Enumerator
                SyncLock Me.m_root
                    Return m_collection.GetEnumerator()
                End SyncLock
            End Function
#End Region

#Region "Public Helpers"
            '// (just to mimic some nice features of ArrayList)
            Public Overrides Property Capacity() As Integer
                Get
                    SyncLock Me.m_root
                        Return m_collection.Capacity
                    End SyncLock
                End Get

                Set(ByVal Value As Integer)
                    SyncLock Me.m_root
                        m_collection.Capacity = Value
                    End SyncLock
                End Set
            End Property
            
            Public Overloads Overrides Function AddRange(ByVal x As <xsl:value-of select="collectionName"/>) As Integer
                SyncLock Me.m_root
                    Return m_collection.AddRange(x)
                End SyncLock
            End Function
            
            Public Overloads Overrides Function AddRange(ByVal x() As <xsl:value-of select="itemType"/>) As Integer
                SyncLock Me.m_root
                    Return m_collection.AddRange(x)
                End SyncLock
            End Function
#End Region
        End Class
#End Region

#Region "Nested Read Only Wrapper class"
        Private Class ReadOnly<xsl:value-of select="collectionName"/>
            Inherits <xsl:value-of select="collectionName"/>

#Region "Implementation (data)"
            Private m_collection As <xsl:value-of select="collectionName"/>
#End Region

#Region "Construction"
            Friend Sub New(ByVal list As <xsl:value-of select="collectionName"/>)
                m_collection = list
            End Sub
#End Region

#Region "Type-safe ICollection"
            
            Public Overloads Overrides Sub CopyTo(ByVal array() As <xsl:value-of select="itemType"/>)
                m_collection.CopyTo(array)
            End Sub

            Public Overloads Overrides Sub CopyTo(ByVal array() As <xsl:value-of select="itemType"/>, ByVal start As Integer)
                m_collection.CopyTo(array, start)
            End Sub

            Public Overrides ReadOnly Property Count() As Integer
                Get
                    Return m_collection.Count
                End Get
            End Property

            Public Overrides ReadOnly Property IsSynchronized() As Boolean
                Get
                    Return m_collection.IsSynchronized
                End Get
            End Property

            Public Overrides ReadOnly Property SyncRoot() As Object
                Get
                    Return Me.m_collection.SyncRoot
                End Get
            End Property
#End Region

#Region "Type-safe IList"
            Default Public Overrides Property Item(ByVal i As Integer) As <xsl:value-of select="itemType"/>
                Get
                    Return m_collection(i)
                End Get

                Set(ByVal Value As <xsl:value-of select="itemType"/>)
                    Throw New NotSupportedException("This is a Read Only Collection and can not be modified")
                End Set
            End Property
            
            Public Overloads Overrides Function Add(ByVal x As <xsl:value-of select="itemType"/>) As Integer
                Throw New NotSupportedException("This is a Read Only Collection and can not be modified")
            End Function

            Public Overrides Sub Clear()
                Throw New NotSupportedException("This is a Read Only Collection and can not be modified")
            End Sub
            
            Public Overloads Overrides Function Contains(ByVal x As <xsl:value-of select="itemType"/>) As Boolean
                Return m_collection.Contains(x)
            End Function
            
            Public Overloads Overrides Function IndexOf(ByVal x As <xsl:value-of select="itemType"/>) As Integer
                Return m_collection.IndexOf(x)
            End Function
            
            Public Overloads Overrides Sub Insert(ByVal pos As Integer, ByVal x As <xsl:value-of select="itemType"/>)
                Throw New NotSupportedException("This is a Read Only Collection and can not be modified")
            End Sub
            
            Public Overloads Overrides Sub Remove(ByVal x As <xsl:value-of select="itemType"/>)
                Throw New NotSupportedException("This is a Read Only Collection and can not be modified")
            End Sub

            Public Overrides Sub RemoveAt(ByVal pos As Integer)
                Throw New NotSupportedException("This is a Read Only Collection and can not be modified")
            End Sub

            Public Overrides ReadOnly Property IsFixedSize() As Boolean
                Get
                    Return True
                End Get
            End Property

            Public Overrides ReadOnly Property IsReadOnly() As Boolean
                Get
                    Return True
                End Get
            End Property
#End Region

#Region "Type-safe IEnumerable"
            Public Overrides Function GetEnumerator() As I<xsl:value-of select="collectionName"/>Enumerator
                Return m_collection.GetEnumerator()
            End Function
#End Region

#Region "Public Helpers"
            '// (just to mimic some nice features of ArrayList)
            Public Overrides Property Capacity() As Integer
                Get
                    Return m_collection.Capacity
                End Get

                Set(ByVal Value As Integer)
                    Throw New NotSupportedException("This is a Read Only Collection and can not be modified")
                End Set
            End Property
            
            Public Overloads Overrides Function AddRange(ByVal x As <xsl:value-of select="collectionName"/>) As Integer
                Throw New NotSupportedException("This is a Read Only Collection and can not be modified")
            End Function
            
            Public Overloads Overrides Function AddRange(ByVal x() As <xsl:value-of select="itemType"/>) As Integer
                Throw New NotSupportedException("This is a Read Only Collection and can not be modified")
            End Function
#End Region
        End Class
#End Region

    End Class
<xsl:if test="collectionNamespace">
	<xsl:if test="string-length(collectionNamespace) > 0">
End Namespace
	</xsl:if>
</xsl:if>

  </xsl:template>

</xsl:transform>
